from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.session_location_dto_power_type import SessionLocationDtoPowerType
from typing import Union

if TYPE_CHECKING:
    from ..models.session_geo_location_dto import SessionGeoLocationDto


T = TypeVar("T", bound="SessionLocationDto")


@_attrs_define
class SessionLocationDto:
    """
    Attributes:
        id (Union[Unset, str]):
        evse_id (Union[Unset, str]):
        power_type (Union[Unset, SessionLocationDtoPowerType]):  Default: SessionLocationDtoPowerType.AC_1_PHASE.
        country_code (Union[Unset, str]):
        party_id (Union[Unset, str]):
        name (Union[Unset, str]):
        house_number (Union[Unset, str]):
        street (Union[Unset, str]):
        city (Union[Unset, str]):
        postal_code (Union[Unset, str]):
        state (Union[Unset, str]):
        country (Union[Unset, str]):
        hotline_phonenumber (Union[Unset, str]):
        coordinates (Union[Unset, SessionGeoLocationDto]):
        time_zone (Union[Unset, str]):
        has_reimbursement (Union[Unset, bool]):
    """

    id: Union[Unset, str] = UNSET
    evse_id: Union[Unset, str] = UNSET
    power_type: Union[Unset, SessionLocationDtoPowerType] = (
        SessionLocationDtoPowerType.AC_1_PHASE
    )
    country_code: Union[Unset, str] = UNSET
    party_id: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    house_number: Union[Unset, str] = UNSET
    street: Union[Unset, str] = UNSET
    city: Union[Unset, str] = UNSET
    postal_code: Union[Unset, str] = UNSET
    state: Union[Unset, str] = UNSET
    country: Union[Unset, str] = UNSET
    hotline_phonenumber: Union[Unset, str] = UNSET
    coordinates: Union[Unset, "SessionGeoLocationDto"] = UNSET
    time_zone: Union[Unset, str] = UNSET
    has_reimbursement: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        evse_id = self.evse_id

        power_type: Union[Unset, str] = UNSET
        if not isinstance(self.power_type, Unset):
            power_type = self.power_type.value

        country_code = self.country_code

        party_id = self.party_id

        name = self.name

        house_number = self.house_number

        street = self.street

        city = self.city

        postal_code = self.postal_code

        state = self.state

        country = self.country

        hotline_phonenumber = self.hotline_phonenumber

        coordinates: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.coordinates, Unset):
            coordinates = self.coordinates.to_dict()

        time_zone = self.time_zone

        has_reimbursement = self.has_reimbursement

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if evse_id is not UNSET:
            field_dict["evseId"] = evse_id
        if power_type is not UNSET:
            field_dict["powerType"] = power_type
        if country_code is not UNSET:
            field_dict["country_code"] = country_code
        if party_id is not UNSET:
            field_dict["party_id"] = party_id
        if name is not UNSET:
            field_dict["name"] = name
        if house_number is not UNSET:
            field_dict["houseNumber"] = house_number
        if street is not UNSET:
            field_dict["street"] = street
        if city is not UNSET:
            field_dict["city"] = city
        if postal_code is not UNSET:
            field_dict["postal_code"] = postal_code
        if state is not UNSET:
            field_dict["state"] = state
        if country is not UNSET:
            field_dict["country"] = country
        if hotline_phonenumber is not UNSET:
            field_dict["hotline_phonenumber"] = hotline_phonenumber
        if coordinates is not UNSET:
            field_dict["coordinates"] = coordinates
        if time_zone is not UNSET:
            field_dict["time_zone"] = time_zone
        if has_reimbursement is not UNSET:
            field_dict["hasReimbursement"] = has_reimbursement

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.session_geo_location_dto import SessionGeoLocationDto

        d = src_dict.copy()
        id = d.pop("id", UNSET)

        evse_id = d.pop("evseId", UNSET)

        _power_type = d.pop("powerType", UNSET)
        power_type: Union[Unset, SessionLocationDtoPowerType]
        if isinstance(_power_type, Unset) or _power_type is None:
            power_type = UNSET
        else:
            power_type = SessionLocationDtoPowerType(_power_type)

        country_code = d.pop("country_code", UNSET)

        party_id = d.pop("party_id", UNSET)

        name = d.pop("name", UNSET)

        house_number = d.pop("houseNumber", UNSET)

        street = d.pop("street", UNSET)

        city = d.pop("city", UNSET)

        postal_code = d.pop("postal_code", UNSET)

        state = d.pop("state", UNSET)

        country = d.pop("country", UNSET)

        hotline_phonenumber = d.pop("hotline_phonenumber", UNSET)

        _coordinates = d.pop("coordinates", UNSET)
        coordinates: Union[Unset, SessionGeoLocationDto]
        if isinstance(_coordinates, Unset) or _coordinates is None:
            coordinates = UNSET
        else:
            coordinates = SessionGeoLocationDto.from_dict(_coordinates)

        time_zone = d.pop("time_zone", UNSET)

        has_reimbursement = d.pop("hasReimbursement", UNSET)

        session_location_dto = cls(
            id=id,
            evse_id=evse_id,
            power_type=power_type,
            country_code=country_code,
            party_id=party_id,
            name=name,
            house_number=house_number,
            street=street,
            city=city,
            postal_code=postal_code,
            state=state,
            country=country,
            hotline_phonenumber=hotline_phonenumber,
            coordinates=coordinates,
            time_zone=time_zone,
            has_reimbursement=has_reimbursement,
        )

        session_location_dto.additional_properties = d
        return session_location_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from dateutil.parser import isoparse
import datetime
from typing import Union


T = TypeVar("T", bound="GetDiagnosticsRequest")


@_attrs_define
class GetDiagnosticsRequest:
    """
    Attributes:
        location (str):
        retries (Union[Unset, int]):
        retry_interval (Union[Unset, int]):
        start_time (Union[Unset, datetime.datetime]):
        stop_time (Union[Unset, datetime.datetime]):
    """

    location: str
    retries: Union[Unset, int] = UNSET
    retry_interval: Union[Unset, int] = UNSET
    start_time: Union[Unset, datetime.datetime] = UNSET
    stop_time: Union[Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        location = self.location

        retries = self.retries

        retry_interval = self.retry_interval

        start_time: Union[Unset, str] = UNSET
        if not isinstance(self.start_time, Unset):
            start_time = self.start_time.isoformat()

        stop_time: Union[Unset, str] = UNSET
        if not isinstance(self.stop_time, Unset):
            stop_time = self.stop_time.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "location": location,
            }
        )
        if retries is not UNSET:
            field_dict["retries"] = retries
        if retry_interval is not UNSET:
            field_dict["retryInterval"] = retry_interval
        if start_time is not UNSET:
            field_dict["startTime"] = start_time
        if stop_time is not UNSET:
            field_dict["stopTime"] = stop_time

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        location = d.pop("location")

        retries = d.pop("retries", UNSET)

        retry_interval = d.pop("retryInterval", UNSET)

        _start_time = d.pop("startTime", UNSET)
        start_time: Union[Unset, datetime.datetime]
        if isinstance(_start_time, Unset) or _start_time is None:
            start_time = UNSET
        else:
            start_time = isoparse(_start_time)

        _stop_time = d.pop("stopTime", UNSET)
        stop_time: Union[Unset, datetime.datetime]
        if isinstance(_stop_time, Unset) or _stop_time is None:
            stop_time = UNSET
        else:
            stop_time = isoparse(_stop_time)

        get_diagnostics_request = cls(
            location=location,
            retries=retries,
            retry_interval=retry_interval,
            start_time=start_time,
            stop_time=stop_time,
        )

        get_diagnostics_request.additional_properties = d
        return get_diagnostics_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from dateutil.parser import isoparse
import datetime
from ..models.chargepoint_status_dto_connectivity_status import (
    ChargepointStatusDtoConnectivityStatus,
)
from typing import Union

if TYPE_CHECKING:
    from ..models.connector_operational_status_dto import ConnectorOperationalStatusDto


T = TypeVar("T", bound="ChargepointStatusDto")


@_attrs_define
class ChargepointStatusDto:
    """
    Attributes:
        id (Union[Unset, str]):
        display_name (Union[Unset, str]):
        tenant_id (Union[Unset, str]):
        ou (Union[Unset, str]):
        ou_id (Union[Unset, str]):
        ou_name (Union[Unset, str]):
        timestamp (Union[Unset, datetime.datetime]):
        connectivity_status (Union[Unset, ChargepointStatusDtoConnectivityStatus]):  Default:
            ChargepointStatusDtoConnectivityStatus.ONLINE.
        connectors (Union[Unset, List['ConnectorOperationalStatusDto']]):
        websocket_connected (Union[Unset, datetime.datetime]):
        websocket_disconnected (Union[Unset, datetime.datetime]):
    """

    id: Union[Unset, str] = UNSET
    display_name: Union[Unset, str] = UNSET
    tenant_id: Union[Unset, str] = UNSET
    ou: Union[Unset, str] = UNSET
    ou_id: Union[Unset, str] = UNSET
    ou_name: Union[Unset, str] = UNSET
    timestamp: Union[Unset, datetime.datetime] = UNSET
    connectivity_status: Union[Unset, ChargepointStatusDtoConnectivityStatus] = (
        ChargepointStatusDtoConnectivityStatus.ONLINE
    )
    connectors: Union[Unset, List["ConnectorOperationalStatusDto"]] = UNSET
    websocket_connected: Union[Unset, datetime.datetime] = UNSET
    websocket_disconnected: Union[Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        display_name = self.display_name

        tenant_id = self.tenant_id

        ou = self.ou

        ou_id = self.ou_id

        ou_name = self.ou_name

        timestamp: Union[Unset, str] = UNSET
        if not isinstance(self.timestamp, Unset):
            timestamp = self.timestamp.isoformat()

        connectivity_status: Union[Unset, str] = UNSET
        if not isinstance(self.connectivity_status, Unset):
            connectivity_status = self.connectivity_status.value

        connectors: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.connectors, Unset):
            connectors = []
            for connectors_item_data in self.connectors:
                connectors_item = connectors_item_data.to_dict()
                connectors.append(connectors_item)

        websocket_connected: Union[Unset, str] = UNSET
        if not isinstance(self.websocket_connected, Unset):
            websocket_connected = self.websocket_connected.isoformat()

        websocket_disconnected: Union[Unset, str] = UNSET
        if not isinstance(self.websocket_disconnected, Unset):
            websocket_disconnected = self.websocket_disconnected.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if display_name is not UNSET:
            field_dict["displayName"] = display_name
        if tenant_id is not UNSET:
            field_dict["tenantId"] = tenant_id
        if ou is not UNSET:
            field_dict["ou"] = ou
        if ou_id is not UNSET:
            field_dict["ouId"] = ou_id
        if ou_name is not UNSET:
            field_dict["ouName"] = ou_name
        if timestamp is not UNSET:
            field_dict["timestamp"] = timestamp
        if connectivity_status is not UNSET:
            field_dict["connectivityStatus"] = connectivity_status
        if connectors is not UNSET:
            field_dict["connectors"] = connectors
        if websocket_connected is not UNSET:
            field_dict["websocketConnected"] = websocket_connected
        if websocket_disconnected is not UNSET:
            field_dict["websocketDisconnected"] = websocket_disconnected

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.connector_operational_status_dto import (
            ConnectorOperationalStatusDto,
        )

        d = src_dict.copy()
        id = d.pop("id", UNSET)

        display_name = d.pop("displayName", UNSET)

        tenant_id = d.pop("tenantId", UNSET)

        ou = d.pop("ou", UNSET)

        ou_id = d.pop("ouId", UNSET)

        ou_name = d.pop("ouName", UNSET)

        _timestamp = d.pop("timestamp", UNSET)
        timestamp: Union[Unset, datetime.datetime]
        if isinstance(_timestamp, Unset) or _timestamp is None:
            timestamp = UNSET
        else:
            timestamp = isoparse(_timestamp)

        _connectivity_status = d.pop("connectivityStatus", UNSET)
        connectivity_status: Union[Unset, ChargepointStatusDtoConnectivityStatus]
        if isinstance(_connectivity_status, Unset) or _connectivity_status is None:
            connectivity_status = UNSET
        else:
            connectivity_status = ChargepointStatusDtoConnectivityStatus(
                _connectivity_status
            )

        connectors = []
        _connectors = d.pop("connectors", UNSET)
        for connectors_item_data in _connectors or []:
            connectors_item = ConnectorOperationalStatusDto.from_dict(
                connectors_item_data
            )

            connectors.append(connectors_item)

        _websocket_connected = d.pop("websocketConnected", UNSET)
        websocket_connected: Union[Unset, datetime.datetime]
        if isinstance(_websocket_connected, Unset) or _websocket_connected is None:
            websocket_connected = UNSET
        else:
            websocket_connected = isoparse(_websocket_connected)

        _websocket_disconnected = d.pop("websocketDisconnected", UNSET)
        websocket_disconnected: Union[Unset, datetime.datetime]
        if isinstance(_websocket_disconnected, Unset) or _websocket_disconnected is None:
            websocket_disconnected = UNSET
        else:
            websocket_disconnected = isoparse(_websocket_disconnected)

        chargepoint_status_dto = cls(
            id=id,
            display_name=display_name,
            tenant_id=tenant_id,
            ou=ou,
            ou_id=ou_id,
            ou_name=ou_name,
            timestamp=timestamp,
            connectivity_status=connectivity_status,
            connectors=connectors,
            websocket_connected=websocket_connected,
            websocket_disconnected=websocket_disconnected,
        )

        chargepoint_status_dto.additional_properties = d
        return chargepoint_status_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.trigger_message_request_requested_message import (
    TriggerMessageRequestRequestedMessage,
)
from typing import Union


T = TypeVar("T", bound="TriggerMessageRequest")


@_attrs_define
class TriggerMessageRequest:
    """
    Attributes:
        requested_message (TriggerMessageRequestRequestedMessage):  Default:
            TriggerMessageRequestRequestedMessage.BOOTNOTIFICATION.
        connector_id (Union[Unset, int]):
    """

    requested_message: TriggerMessageRequestRequestedMessage = (
        TriggerMessageRequestRequestedMessage.BOOTNOTIFICATION
    )
    connector_id: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        requested_message = self.requested_message.value

        connector_id = self.connector_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "requestedMessage": requested_message,
            }
        )
        if connector_id is not UNSET:
            field_dict["connectorId"] = connector_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        requested_message = TriggerMessageRequestRequestedMessage(
            d.pop("requestedMessage")
        )

        connector_id = d.pop("connectorId", UNSET)

        trigger_message_request = cls(
            requested_message=requested_message,
            connector_id=connector_id,
        )

        trigger_message_request.additional_properties = d
        return trigger_message_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from dateutil.parser import isoparse
import datetime
from typing import Union
from ..models.connector_operational_status_dto_operational_status import (
    ConnectorOperationalStatusDtoOperationalStatus,
)


T = TypeVar("T", bound="ConnectorOperationalStatusDto")


@_attrs_define
class ConnectorOperationalStatusDto:
    """
    Attributes:
        connector_number (Union[Unset, int]):
        operational_status (Union[Unset, ConnectorOperationalStatusDtoOperationalStatus]):  Default:
            ConnectorOperationalStatusDtoOperationalStatus.AVAILABLE.
        timestamp (Union[Unset, datetime.datetime]):
    """

    connector_number: Union[Unset, int] = UNSET
    operational_status: Union[Unset, ConnectorOperationalStatusDtoOperationalStatus] = (
        ConnectorOperationalStatusDtoOperationalStatus.AVAILABLE
    )
    timestamp: Union[Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        connector_number = self.connector_number

        operational_status: Union[Unset, str] = UNSET
        if not isinstance(self.operational_status, Unset):
            operational_status = self.operational_status.value

        timestamp: Union[Unset, str] = UNSET
        if not isinstance(self.timestamp, Unset):
            timestamp = self.timestamp.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if connector_number is not UNSET:
            field_dict["connectorNumber"] = connector_number
        if operational_status is not UNSET:
            field_dict["operationalStatus"] = operational_status
        if timestamp is not UNSET:
            field_dict["timestamp"] = timestamp

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        connector_number = d.pop("connectorNumber", UNSET)

        _operational_status = d.pop("operationalStatus", UNSET)
        operational_status: Union[Unset, ConnectorOperationalStatusDtoOperationalStatus]
        if isinstance(_operational_status, Unset) or _operational_status is None:
            operational_status = UNSET
        else:
            operational_status = ConnectorOperationalStatusDtoOperationalStatus(
                _operational_status
            )

        _timestamp = d.pop("timestamp", UNSET)
        timestamp: Union[Unset, datetime.datetime]
        if isinstance(_timestamp, Unset) or _timestamp is None:
            timestamp = UNSET
        else:
            timestamp = isoparse(_timestamp)

        connector_operational_status_dto = cls(
            connector_number=connector_number,
            operational_status=operational_status,
            timestamp=timestamp,
        )

        connector_operational_status_dto.additional_properties = d
        return connector_operational_status_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class LocationDtoFacilitiesItem(str, Enum):
    ACCOMMODATION = "ACCOMMODATION"
    AIRPORT = "AIRPORT"
    BIKE_SHARING = "BIKE_SHARING"
    BUS_STOP = "BUS_STOP"
    CAFE = "CAFE"
    CARPOOL_PARKING = "CARPOOL_PARKING"
    FUEL_STATION = "FUEL_STATION"
    MALL = "MALL"
    METRO_STATION = "METRO_STATION"
    MUSEUM = "MUSEUM"
    NATURE = "NATURE"
    PARKING_LOT = "PARKING_LOT"
    RECREATION_AREA = "RECREATION_AREA"
    RESTAURANT = "RESTAURANT"
    SPORT = "SPORT"
    SUPERMARKET = "SUPERMARKET"
    TAXI_STAND = "TAXI_STAND"
    TRAIN_STATION = "TRAIN_STATION"
    TRAM_STOP = "TRAM_STOP"
    WIFI = "WIFI"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
from ..models.cdr_patch_dto_approval_status import CdrPatchDtoApprovalStatus


T = TypeVar("T", bound="CdrPatchDto")


@_attrs_define
class CdrPatchDto:
    """
    Attributes:
        approval_status (Union[Unset, CdrPatchDtoApprovalStatus]):  Default: CdrPatchDtoApprovalStatus.APPROVED.
    """

    approval_status: Union[Unset, CdrPatchDtoApprovalStatus] = (
        CdrPatchDtoApprovalStatus.APPROVED
    )
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        approval_status: Union[Unset, str] = UNSET
        if not isinstance(self.approval_status, Unset):
            approval_status = self.approval_status.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if approval_status is not UNSET:
            field_dict["approvalStatus"] = approval_status

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _approval_status = d.pop("approvalStatus", UNSET)
        approval_status: Union[Unset, CdrPatchDtoApprovalStatus]
        if isinstance(_approval_status, Unset) or _approval_status is None:
            approval_status = UNSET
        else:
            approval_status = CdrPatchDtoApprovalStatus(_approval_status)

        cdr_patch_dto = cls(
            approval_status=approval_status,
        )

        cdr_patch_dto.additional_properties = d
        return cdr_patch_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union


T = TypeVar("T", bound="ChargingSchedulePeriod")


@_attrs_define
class ChargingSchedulePeriod:
    """
    Attributes:
        start_period (int):
        limit (float):
        number_phases (Union[Unset, int]):
    """

    start_period: int
    limit: float
    number_phases: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        start_period = self.start_period

        limit = self.limit

        number_phases = self.number_phases

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "startPeriod": start_period,
                "limit": limit,
            }
        )
        if number_phases is not UNSET:
            field_dict["numberPhases"] = number_phases

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        start_period = d.pop("startPeriod")

        limit = d.pop("limit")

        number_phases = d.pop("numberPhases", UNSET)

        charging_schedule_period = cls(
            start_period=start_period,
            limit=limit,
            number_phases=number_phases,
        )

        charging_schedule_period.additional_properties = d
        return charging_schedule_period

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ImageDtoCategory(str, Enum):
    CHARGER = "CHARGER"
    ENTRANCE = "ENTRANCE"
    LOCATION = "LOCATION"
    NETWORK = "NETWORK"
    OPERATOR = "OPERATOR"
    OTHER = "OTHER"
    OWNER = "OWNER"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from dateutil.parser import isoparse
from ..models.location_dto_facilities_item import LocationDtoFacilitiesItem
import datetime
from ..models.location_dto_parking_type import LocationDtoParkingType
from typing import Union

if TYPE_CHECKING:
    from ..models.display_text_dto import DisplayTextDto
    from ..models.reimburse_info_dto import ReimburseInfoDto
    from ..models.publish_token_type_dto import PublishTokenTypeDto
    from ..models.image_dto import ImageDto
    from ..models.hours_dto import HoursDto
    from ..models.energy_mix_dto import EnergyMixDto
    from ..models.geo_location_dto import GeoLocationDto
    from ..models.business_details_dto import BusinessDetailsDto
    from ..models.location_evse_dto import LocationEVSEDto
    from ..models.additional_geo_location_dto import AdditionalGeoLocationDto


T = TypeVar("T", bound="LocationDto")


@_attrs_define
class LocationDto:
    """
    Attributes:
        id (str):
        country_code (str):
        party_id (str):
        publish (bool):
        street (str):
        city (str):
        country (str):
        coordinates (GeoLocationDto):
        time_zone (str):
        last_updated (datetime.datetime):
        publish_allowed_to (Union[Unset, List['PublishTokenTypeDto']]):
        name (Union[Unset, str]):
        house_number (Union[Unset, str]):
        postal_code (Union[Unset, str]):
        state (Union[Unset, str]):
        hotline_phonenumber (Union[Unset, str]):
        related_locations (Union[Unset, List['AdditionalGeoLocationDto']]):
        parking_type (Union[Unset, LocationDtoParkingType]):  Default: LocationDtoParkingType.ALONG_MOTORWAY.
        evses (Union[Unset, List['LocationEVSEDto']]):
        directions (Union[Unset, List['DisplayTextDto']]):
        operator (Union[Unset, BusinessDetailsDto]):
        suboperator (Union[Unset, BusinessDetailsDto]):
        owner (Union[Unset, BusinessDetailsDto]):
        facilities (Union[Unset, List[LocationDtoFacilitiesItem]]):
        opening_times (Union[Unset, HoursDto]):
        charging_when_closed (Union[Unset, bool]):
        images (Union[Unset, List['ImageDto']]):
        energy_mix (Union[Unset, EnergyMixDto]):
        deleted_on (Union[Unset, datetime.datetime]):
        has_reimbursement (Union[Unset, bool]):
        reimburse_info (Union[Unset, ReimburseInfoDto]):
        operator_id (Union[Unset, str]):
        ou (Union[Unset, str]):
        ou_id (Union[Unset, str]):
        ou_name (Union[Unset, str]):
        external_reference_1 (Union[Unset, str]):
        external_reference_2 (Union[Unset, str]):
        external_reference_3 (Union[Unset, str]):
    """

    id: str
    country_code: str
    party_id: str
    publish: bool
    street: str
    city: str
    country: str
    coordinates: "GeoLocationDto"
    time_zone: str
    last_updated: datetime.datetime
    publish_allowed_to: Union[Unset, List["PublishTokenTypeDto"]] = UNSET
    name: Union[Unset, str] = UNSET
    house_number: Union[Unset, str] = UNSET
    postal_code: Union[Unset, str] = UNSET
    state: Union[Unset, str] = UNSET
    hotline_phonenumber: Union[Unset, str] = UNSET
    related_locations: Union[Unset, List["AdditionalGeoLocationDto"]] = UNSET
    parking_type: Union[Unset, LocationDtoParkingType] = (
        LocationDtoParkingType.ALONG_MOTORWAY
    )
    evses: Union[Unset, List["LocationEVSEDto"]] = UNSET
    directions: Union[Unset, List["DisplayTextDto"]] = UNSET
    operator: Union[Unset, "BusinessDetailsDto"] = UNSET
    suboperator: Union[Unset, "BusinessDetailsDto"] = UNSET
    owner: Union[Unset, "BusinessDetailsDto"] = UNSET
    facilities: Union[Unset, List[LocationDtoFacilitiesItem]] = UNSET
    opening_times: Union[Unset, "HoursDto"] = UNSET
    charging_when_closed: Union[Unset, bool] = UNSET
    images: Union[Unset, List["ImageDto"]] = UNSET
    energy_mix: Union[Unset, "EnergyMixDto"] = UNSET
    deleted_on: Union[Unset, datetime.datetime] = UNSET
    has_reimbursement: Union[Unset, bool] = UNSET
    reimburse_info: Union[Unset, "ReimburseInfoDto"] = UNSET
    operator_id: Union[Unset, str] = UNSET
    ou: Union[Unset, str] = UNSET
    ou_id: Union[Unset, str] = UNSET
    ou_name: Union[Unset, str] = UNSET
    external_reference_1: Union[Unset, str] = UNSET
    external_reference_2: Union[Unset, str] = UNSET
    external_reference_3: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        country_code = self.country_code

        party_id = self.party_id

        publish = self.publish

        street = self.street

        city = self.city

        country = self.country

        coordinates = self.coordinates.to_dict()

        time_zone = self.time_zone

        last_updated = self.last_updated.isoformat()

        publish_allowed_to: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.publish_allowed_to, Unset):
            publish_allowed_to = []
            for publish_allowed_to_item_data in self.publish_allowed_to:
                publish_allowed_to_item = publish_allowed_to_item_data.to_dict()
                publish_allowed_to.append(publish_allowed_to_item)

        name = self.name

        house_number = self.house_number

        postal_code = self.postal_code

        state = self.state

        hotline_phonenumber = self.hotline_phonenumber

        related_locations: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.related_locations, Unset):
            related_locations = []
            for related_locations_item_data in self.related_locations:
                related_locations_item = related_locations_item_data.to_dict()
                related_locations.append(related_locations_item)

        parking_type: Union[Unset, str] = UNSET
        if not isinstance(self.parking_type, Unset):
            parking_type = self.parking_type.value

        evses: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.evses, Unset):
            evses = []
            for evses_item_data in self.evses:
                evses_item = evses_item_data.to_dict()
                evses.append(evses_item)

        directions: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.directions, Unset):
            directions = []
            for directions_item_data in self.directions:
                directions_item = directions_item_data.to_dict()
                directions.append(directions_item)

        operator: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.operator, Unset):
            operator = self.operator.to_dict()

        suboperator: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.suboperator, Unset):
            suboperator = self.suboperator.to_dict()

        owner: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.owner, Unset):
            owner = self.owner.to_dict()

        facilities: Union[Unset, List[str]] = UNSET
        if not isinstance(self.facilities, Unset):
            facilities = []
            for facilities_item_data in self.facilities:
                facilities_item = facilities_item_data.value
                facilities.append(facilities_item)

        opening_times: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.opening_times, Unset):
            opening_times = self.opening_times.to_dict()

        charging_when_closed = self.charging_when_closed

        images: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.images, Unset):
            images = []
            for images_item_data in self.images:
                images_item = images_item_data.to_dict()
                images.append(images_item)

        energy_mix: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.energy_mix, Unset):
            energy_mix = self.energy_mix.to_dict()

        deleted_on: Union[Unset, str] = UNSET
        if not isinstance(self.deleted_on, Unset):
            deleted_on = self.deleted_on.isoformat()

        has_reimbursement = self.has_reimbursement

        reimburse_info: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.reimburse_info, Unset):
            reimburse_info = self.reimburse_info.to_dict()

        operator_id = self.operator_id

        ou = self.ou

        ou_id = self.ou_id

        ou_name = self.ou_name

        external_reference_1 = self.external_reference_1

        external_reference_2 = self.external_reference_2

        external_reference_3 = self.external_reference_3

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "country_code": country_code,
                "party_id": party_id,
                "publish": publish,
                "street": street,
                "city": city,
                "country": country,
                "coordinates": coordinates,
                "time_zone": time_zone,
                "last_updated": last_updated,
            }
        )
        if publish_allowed_to is not UNSET:
            field_dict["publish_allowed_to"] = publish_allowed_to
        if name is not UNSET:
            field_dict["name"] = name
        if house_number is not UNSET:
            field_dict["houseNumber"] = house_number
        if postal_code is not UNSET:
            field_dict["postal_code"] = postal_code
        if state is not UNSET:
            field_dict["state"] = state
        if hotline_phonenumber is not UNSET:
            field_dict["hotline_phonenumber"] = hotline_phonenumber
        if related_locations is not UNSET:
            field_dict["related_locations"] = related_locations
        if parking_type is not UNSET:
            field_dict["parking_type"] = parking_type
        if evses is not UNSET:
            field_dict["evses"] = evses
        if directions is not UNSET:
            field_dict["directions"] = directions
        if operator is not UNSET:
            field_dict["operator"] = operator
        if suboperator is not UNSET:
            field_dict["suboperator"] = suboperator
        if owner is not UNSET:
            field_dict["owner"] = owner
        if facilities is not UNSET:
            field_dict["facilities"] = facilities
        if opening_times is not UNSET:
            field_dict["opening_times"] = opening_times
        if charging_when_closed is not UNSET:
            field_dict["charging_when_closed"] = charging_when_closed
        if images is not UNSET:
            field_dict["images"] = images
        if energy_mix is not UNSET:
            field_dict["energy_mix"] = energy_mix
        if deleted_on is not UNSET:
            field_dict["deletedOn"] = deleted_on
        if has_reimbursement is not UNSET:
            field_dict["hasReimbursement"] = has_reimbursement
        if reimburse_info is not UNSET:
            field_dict["reimburseInfo"] = reimburse_info
        if operator_id is not UNSET:
            field_dict["operatorId"] = operator_id
        if ou is not UNSET:
            field_dict["ou"] = ou
        if ou_id is not UNSET:
            field_dict["ouId"] = ou_id
        if ou_name is not UNSET:
            field_dict["ouName"] = ou_name
        if external_reference_1 is not UNSET:
            field_dict["externalReference1"] = external_reference_1
        if external_reference_2 is not UNSET:
            field_dict["externalReference2"] = external_reference_2
        if external_reference_3 is not UNSET:
            field_dict["externalReference3"] = external_reference_3

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.display_text_dto import DisplayTextDto
        from ..models.reimburse_info_dto import ReimburseInfoDto
        from ..models.publish_token_type_dto import PublishTokenTypeDto
        from ..models.image_dto import ImageDto
        from ..models.hours_dto import HoursDto
        from ..models.energy_mix_dto import EnergyMixDto
        from ..models.geo_location_dto import GeoLocationDto
        from ..models.business_details_dto import BusinessDetailsDto
        from ..models.location_evse_dto import LocationEVSEDto
        from ..models.additional_geo_location_dto import AdditionalGeoLocationDto

        d = src_dict.copy()
        id = d.pop("id")

        country_code = d.pop("country_code")

        party_id = d.pop("party_id")

        publish = d.pop("publish")

        street = d.pop("street")

        city = d.pop("city")

        country = d.pop("country")

        coordinates = GeoLocationDto.from_dict(d.pop("coordinates"))

        time_zone = d.pop("time_zone")

        last_updated = isoparse(d.pop("last_updated"))

        publish_allowed_to = []
        _publish_allowed_to = d.pop("publish_allowed_to", UNSET)
        for publish_allowed_to_item_data in _publish_allowed_to or []:
            publish_allowed_to_item = PublishTokenTypeDto.from_dict(
                publish_allowed_to_item_data
            )

            publish_allowed_to.append(publish_allowed_to_item)

        name = d.pop("name", UNSET)

        house_number = d.pop("houseNumber", UNSET)

        postal_code = d.pop("postal_code", UNSET)

        state = d.pop("state", UNSET)

        hotline_phonenumber = d.pop("hotline_phonenumber", UNSET)

        related_locations = []
        _related_locations = d.pop("related_locations", UNSET)
        for related_locations_item_data in _related_locations or []:
            related_locations_item = AdditionalGeoLocationDto.from_dict(
                related_locations_item_data
            )

            related_locations.append(related_locations_item)

        _parking_type = d.pop("parking_type", UNSET)
        parking_type: Union[Unset, LocationDtoParkingType]
        if isinstance(_parking_type, Unset) or _parking_type is None:
            parking_type = UNSET
        else:
            parking_type = LocationDtoParkingType(_parking_type)

        evses = []
        _evses = d.pop("evses", UNSET)
        for evses_item_data in _evses or []:
            evses_item = LocationEVSEDto.from_dict(evses_item_data)

            evses.append(evses_item)

        directions = []
        _directions = d.pop("directions", UNSET)
        for directions_item_data in _directions or []:
            directions_item = DisplayTextDto.from_dict(directions_item_data)

            directions.append(directions_item)

        _operator = d.pop("operator", UNSET)
        operator: Union[Unset, BusinessDetailsDto]
        if isinstance(_operator, Unset) or _operator is None:
            operator = UNSET
        else:
            operator = BusinessDetailsDto.from_dict(_operator)

        _suboperator = d.pop("suboperator", UNSET)
        suboperator: Union[Unset, BusinessDetailsDto]
        if isinstance(_suboperator, Unset) or _suboperator is None:
            suboperator = UNSET
        else:
            suboperator = BusinessDetailsDto.from_dict(_suboperator)

        _owner = d.pop("owner", UNSET)
        owner: Union[Unset, BusinessDetailsDto]
        if isinstance(_owner, Unset) or _owner is None:
            owner = UNSET
        else:
            owner = BusinessDetailsDto.from_dict(_owner)

        facilities = []
        _facilities = d.pop("facilities", UNSET)
        for facilities_item_data in _facilities or []:
            facilities_item = LocationDtoFacilitiesItem(facilities_item_data)

            facilities.append(facilities_item)

        _opening_times = d.pop("opening_times", UNSET)
        opening_times: Union[Unset, HoursDto]
        if isinstance(_opening_times, Unset) or _opening_times is None:
            opening_times = UNSET
        else:
            opening_times = HoursDto.from_dict(_opening_times)

        charging_when_closed = d.pop("charging_when_closed", UNSET)

        images = []
        _images = d.pop("images", UNSET)
        for images_item_data in _images or []:
            images_item = ImageDto.from_dict(images_item_data)

            images.append(images_item)

        _energy_mix = d.pop("energy_mix", UNSET)
        energy_mix: Union[Unset, EnergyMixDto]
        if isinstance(_energy_mix, Unset) or _energy_mix is None:
            energy_mix = UNSET
        else:
            energy_mix = EnergyMixDto.from_dict(_energy_mix)

        _deleted_on = d.pop("deletedOn", UNSET)
        deleted_on: Union[Unset, datetime.datetime]
        if isinstance(_deleted_on, Unset) or _deleted_on is None:
            deleted_on = UNSET
        else:
            deleted_on = isoparse(_deleted_on)

        has_reimbursement = d.pop("hasReimbursement", UNSET)

        _reimburse_info = d.pop("reimburseInfo", UNSET)
        reimburse_info: Union[Unset, ReimburseInfoDto]
        if isinstance(_reimburse_info, Unset) or _reimburse_info is None:
            reimburse_info = UNSET
        else:
            reimburse_info = ReimburseInfoDto.from_dict(_reimburse_info)

        operator_id = d.pop("operatorId", UNSET)

        ou = d.pop("ou", UNSET)

        ou_id = d.pop("ouId", UNSET)

        ou_name = d.pop("ouName", UNSET)

        external_reference_1 = d.pop("externalReference1", UNSET)

        external_reference_2 = d.pop("externalReference2", UNSET)

        external_reference_3 = d.pop("externalReference3", UNSET)

        location_dto = cls(
            id=id,
            country_code=country_code,
            party_id=party_id,
            publish=publish,
            street=street,
            city=city,
            country=country,
            coordinates=coordinates,
            time_zone=time_zone,
            last_updated=last_updated,
            publish_allowed_to=publish_allowed_to,
            name=name,
            house_number=house_number,
            postal_code=postal_code,
            state=state,
            hotline_phonenumber=hotline_phonenumber,
            related_locations=related_locations,
            parking_type=parking_type,
            evses=evses,
            directions=directions,
            operator=operator,
            suboperator=suboperator,
            owner=owner,
            facilities=facilities,
            opening_times=opening_times,
            charging_when_closed=charging_when_closed,
            images=images,
            energy_mix=energy_mix,
            deleted_on=deleted_on,
            has_reimbursement=has_reimbursement,
            reimburse_info=reimburse_info,
            operator_id=operator_id,
            ou=ou,
            ou_id=ou_id,
            ou_name=ou_name,
            external_reference_1=external_reference_1,
            external_reference_2=external_reference_2,
            external_reference_3=external_reference_3,
        )

        location_dto.additional_properties = d
        return location_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Dict, List, Optional, Union

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...models.get_all_chargepoints_operationalstatus import (
    GetAllChargepointsOperationalstatus,
)
from ...models.chargepoint_dto import ChargepointDto
from ...models.get_all_chargepoints_accesstype import GetAllChargepointsAccesstype
from ...models.get_all_chargepoints_chargerpowertype import (
    GetAllChargepointsChargerpowertype,
)
from ...types import Unset
from ...models.longship_error import LongshipError
from ...models.get_all_chargepoints_order_by import GetAllChargepointsOrderBy


def _get_kwargs(
    *,
    skip: Union[Unset, int] = UNSET,
    take: Union[Unset, int] = UNSET,
    search: Union[Unset, str] = UNSET,
    is_online: Union[Unset, bool] = UNSET,
    tariff_id: Union[Unset, str] = UNSET,
    operationalstatus: Union[
        Unset, GetAllChargepointsOperationalstatus
    ] = GetAllChargepointsOperationalstatus.AVAILABLE,
    accesstype: Union[
        Unset, GetAllChargepointsAccesstype
    ] = GetAllChargepointsAccesstype.NEW,
    chargerpowertype: Union[
        Unset, GetAllChargepointsChargerpowertype
    ] = GetAllChargepointsChargerpowertype.AC,
    organizationunitcode: Union[Unset, str] = UNSET,
    inactiveonly: Union[Unset, bool] = UNSET,
    firmwareversion: Union[Unset, str] = UNSET,
    model: Union[Unset, str] = UNSET,
    vendor: Union[Unset, str] = UNSET,
    order_by: Union[Unset, GetAllChargepointsOrderBy] = GetAllChargepointsOrderBy.NAME,
    descending: Union[Unset, bool] = UNSET,
) -> Dict[str, Any]:
    params: Dict[str, Any] = {}

    params["skip"] = skip

    params["take"] = take

    params["search"] = search

    params["isOnline"] = is_online

    params["tariffId"] = tariff_id

    json_operationalstatus: Union[Unset, str] = UNSET
    if not isinstance(operationalstatus, Unset):
        json_operationalstatus = operationalstatus.value

    params["operationalstatus"] = json_operationalstatus

    json_accesstype: Union[Unset, str] = UNSET
    if not isinstance(accesstype, Unset):
        json_accesstype = accesstype.value

    params["accesstype"] = json_accesstype

    json_chargerpowertype: Union[Unset, str] = UNSET
    if not isinstance(chargerpowertype, Unset):
        json_chargerpowertype = chargerpowertype.value

    params["chargerpowertype"] = json_chargerpowertype

    params["organizationunitcode"] = organizationunitcode

    params["inactiveonly"] = inactiveonly

    params["firmwareversion"] = firmwareversion

    params["model"] = model

    params["vendor"] = vendor

    json_order_by: Union[Unset, str] = UNSET
    if not isinstance(order_by, Unset):
        json_order_by = order_by.value

    params["orderBy"] = json_order_by

    params["descending"] = descending

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": "/v1/chargepoints",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[List["ChargepointDto"], LongshipError]]:
    if response.status_code == HTTPStatus.OK:
        response_200 = []
        _response_200 = response.json()
        for componentsschemaschargepoint_dto_array_item_data in _response_200:
            componentsschemaschargepoint_dto_array_item = ChargepointDto.from_dict(
                componentsschemaschargepoint_dto_array_item_data
            )

            response_200.append(componentsschemaschargepoint_dto_array_item)

        return response_200
    if response.status_code == HTTPStatus.UNAUTHORIZED:
        response_401 = LongshipError.from_dict(response.json())

        return response_401
    if response.status_code == HTTPStatus.INTERNAL_SERVER_ERROR:
        response_500 = LongshipError.from_dict(response.json())

        return response_500
    if response.status_code == HTTPStatus.FORBIDDEN:
        response_403 = LongshipError.from_dict(response.json())

        return response_403
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[List["ChargepointDto"], LongshipError]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    skip: Union[Unset, int] = UNSET,
    take: Union[Unset, int] = UNSET,
    search: Union[Unset, str] = UNSET,
    is_online: Union[Unset, bool] = UNSET,
    tariff_id: Union[Unset, str] = UNSET,
    operationalstatus: Union[
        Unset, GetAllChargepointsOperationalstatus
    ] = GetAllChargepointsOperationalstatus.AVAILABLE,
    accesstype: Union[
        Unset, GetAllChargepointsAccesstype
    ] = GetAllChargepointsAccesstype.NEW,
    chargerpowertype: Union[
        Unset, GetAllChargepointsChargerpowertype
    ] = GetAllChargepointsChargerpowertype.AC,
    organizationunitcode: Union[Unset, str] = UNSET,
    inactiveonly: Union[Unset, bool] = UNSET,
    firmwareversion: Union[Unset, str] = UNSET,
    model: Union[Unset, str] = UNSET,
    vendor: Union[Unset, str] = UNSET,
    order_by: Union[Unset, GetAllChargepointsOrderBy] = GetAllChargepointsOrderBy.NAME,
    descending: Union[Unset, bool] = UNSET,
) -> Response[Union[List["ChargepointDto"], LongshipError]]:
    """Get a list of chargepoints.

     Get a paged list of chargepoints, taken the filters into account.

    Args:
        skip (Union[Unset, int]):
        take (Union[Unset, int]):
        search (Union[Unset, str]):
        is_online (Union[Unset, bool]):
        tariff_id (Union[Unset, str]):
        operationalstatus (Union[Unset, GetAllChargepointsOperationalstatus]):  Default:
            GetAllChargepointsOperationalstatus.AVAILABLE.
        accesstype (Union[Unset, GetAllChargepointsAccesstype]):  Default:
            GetAllChargepointsAccesstype.NEW.
        chargerpowertype (Union[Unset, GetAllChargepointsChargerpowertype]):  Default:
            GetAllChargepointsChargerpowertype.AC.
        organizationunitcode (Union[Unset, str]):
        inactiveonly (Union[Unset, bool]):
        firmwareversion (Union[Unset, str]):
        model (Union[Unset, str]):
        vendor (Union[Unset, str]):
        order_by (Union[Unset, GetAllChargepointsOrderBy]):  Default:
            GetAllChargepointsOrderBy.NAME.
        descending (Union[Unset, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[List['ChargepointDto'], LongshipError]]
    """

    kwargs = _get_kwargs(
        skip=skip,
        take=take,
        search=search,
        is_online=is_online,
        tariff_id=tariff_id,
        operationalstatus=operationalstatus,
        accesstype=accesstype,
        chargerpowertype=chargerpowertype,
        organizationunitcode=organizationunitcode,
        inactiveonly=inactiveonly,
        firmwareversion=firmwareversion,
        model=model,
        vendor=vendor,
        order_by=order_by,
        descending=descending,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: Union[AuthenticatedClient, Client],
    skip: Union[Unset, int] = UNSET,
    take: Union[Unset, int] = UNSET,
    search: Union[Unset, str] = UNSET,
    is_online: Union[Unset, bool] = UNSET,
    tariff_id: Union[Unset, str] = UNSET,
    operationalstatus: Union[
        Unset, GetAllChargepointsOperationalstatus
    ] = GetAllChargepointsOperationalstatus.AVAILABLE,
    accesstype: Union[
        Unset, GetAllChargepointsAccesstype
    ] = GetAllChargepointsAccesstype.NEW,
    chargerpowertype: Union[
        Unset, GetAllChargepointsChargerpowertype
    ] = GetAllChargepointsChargerpowertype.AC,
    organizationunitcode: Union[Unset, str] = UNSET,
    inactiveonly: Union[Unset, bool] = UNSET,
    firmwareversion: Union[Unset, str] = UNSET,
    model: Union[Unset, str] = UNSET,
    vendor: Union[Unset, str] = UNSET,
    order_by: Union[Unset, GetAllChargepointsOrderBy] = GetAllChargepointsOrderBy.NAME,
    descending: Union[Unset, bool] = UNSET,
) -> Optional[Union[List["ChargepointDto"], LongshipError]]:
    """Get a list of chargepoints.

     Get a paged list of chargepoints, taken the filters into account.

    Args:
        skip (Union[Unset, int]):
        take (Union[Unset, int]):
        search (Union[Unset, str]):
        is_online (Union[Unset, bool]):
        tariff_id (Union[Unset, str]):
        operationalstatus (Union[Unset, GetAllChargepointsOperationalstatus]):  Default:
            GetAllChargepointsOperationalstatus.AVAILABLE.
        accesstype (Union[Unset, GetAllChargepointsAccesstype]):  Default:
            GetAllChargepointsAccesstype.NEW.
        chargerpowertype (Union[Unset, GetAllChargepointsChargerpowertype]):  Default:
            GetAllChargepointsChargerpowertype.AC.
        organizationunitcode (Union[Unset, str]):
        inactiveonly (Union[Unset, bool]):
        firmwareversion (Union[Unset, str]):
        model (Union[Unset, str]):
        vendor (Union[Unset, str]):
        order_by (Union[Unset, GetAllChargepointsOrderBy]):  Default:
            GetAllChargepointsOrderBy.NAME.
        descending (Union[Unset, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[List['ChargepointDto'], LongshipError]
    """

    return sync_detailed(
        client=client,
        skip=skip,
        take=take,
        search=search,
        is_online=is_online,
        tariff_id=tariff_id,
        operationalstatus=operationalstatus,
        accesstype=accesstype,
        chargerpowertype=chargerpowertype,
        organizationunitcode=organizationunitcode,
        inactiveonly=inactiveonly,
        firmwareversion=firmwareversion,
        model=model,
        vendor=vendor,
        order_by=order_by,
        descending=descending,
    ).parsed


async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    skip: Union[Unset, int] = UNSET,
    take: Union[Unset, int] = UNSET,
    search: Union[Unset, str] = UNSET,
    is_online: Union[Unset, bool] = UNSET,
    tariff_id: Union[Unset, str] = UNSET,
    operationalstatus: Union[
        Unset, GetAllChargepointsOperationalstatus
    ] = GetAllChargepointsOperationalstatus.AVAILABLE,
    accesstype: Union[
        Unset, GetAllChargepointsAccesstype
    ] = GetAllChargepointsAccesstype.NEW,
    chargerpowertype: Union[
        Unset, GetAllChargepointsChargerpowertype
    ] = GetAllChargepointsChargerpowertype.AC,
    organizationunitcode: Union[Unset, str] = UNSET,
    inactiveonly: Union[Unset, bool] = UNSET,
    firmwareversion: Union[Unset, str] = UNSET,
    model: Union[Unset, str] = UNSET,
    vendor: Union[Unset, str] = UNSET,
    order_by: Union[Unset, GetAllChargepointsOrderBy] = GetAllChargepointsOrderBy.NAME,
    descending: Union[Unset, bool] = UNSET,
) -> Response[Union[List["ChargepointDto"], LongshipError]]:
    """Get a list of chargepoints.

     Get a paged list of chargepoints, taken the filters into account.

    Args:
        skip (Union[Unset, int]):
        take (Union[Unset, int]):
        search (Union[Unset, str]):
        is_online (Union[Unset, bool]):
        tariff_id (Union[Unset, str]):
        operationalstatus (Union[Unset, GetAllChargepointsOperationalstatus]):  Default:
            GetAllChargepointsOperationalstatus.AVAILABLE.
        accesstype (Union[Unset, GetAllChargepointsAccesstype]):  Default:
            GetAllChargepointsAccesstype.NEW.
        chargerpowertype (Union[Unset, GetAllChargepointsChargerpowertype]):  Default:
            GetAllChargepointsChargerpowertype.AC.
        organizationunitcode (Union[Unset, str]):
        inactiveonly (Union[Unset, bool]):
        firmwareversion (Union[Unset, str]):
        model (Union[Unset, str]):
        vendor (Union[Unset, str]):
        order_by (Union[Unset, GetAllChargepointsOrderBy]):  Default:
            GetAllChargepointsOrderBy.NAME.
        descending (Union[Unset, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[List['ChargepointDto'], LongshipError]]
    """

    kwargs = _get_kwargs(
        skip=skip,
        take=take,
        search=search,
        is_online=is_online,
        tariff_id=tariff_id,
        operationalstatus=operationalstatus,
        accesstype=accesstype,
        chargerpowertype=chargerpowertype,
        organizationunitcode=organizationunitcode,
        inactiveonly=inactiveonly,
        firmwareversion=firmwareversion,
        model=model,
        vendor=vendor,
        order_by=order_by,
        descending=descending,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: Union[AuthenticatedClient, Client],
    skip: Union[Unset, int] = UNSET,
    take: Union[Unset, int] = UNSET,
    search: Union[Unset, str] = UNSET,
    is_online: Union[Unset, bool] = UNSET,
    tariff_id: Union[Unset, str] = UNSET,
    operationalstatus: Union[
        Unset, GetAllChargepointsOperationalstatus
    ] = GetAllChargepointsOperationalstatus.AVAILABLE,
    accesstype: Union[
        Unset, GetAllChargepointsAccesstype
    ] = GetAllChargepointsAccesstype.NEW,
    chargerpowertype: Union[
        Unset, GetAllChargepointsChargerpowertype
    ] = GetAllChargepointsChargerpowertype.AC,
    organizationunitcode: Union[Unset, str] = UNSET,
    inactiveonly: Union[Unset, bool] = UNSET,
    firmwareversion: Union[Unset, str] = UNSET,
    model: Union[Unset, str] = UNSET,
    vendor: Union[Unset, str] = UNSET,
    order_by: Union[Unset, GetAllChargepointsOrderBy] = GetAllChargepointsOrderBy.NAME,
    descending: Union[Unset, bool] = UNSET,
) -> Optional[Union[List["ChargepointDto"], LongshipError]]:
    """Get a list of chargepoints.

     Get a paged list of chargepoints, taken the filters into account.

    Args:
        skip (Union[Unset, int]):
        take (Union[Unset, int]):
        search (Union[Unset, str]):
        is_online (Union[Unset, bool]):
        tariff_id (Union[Unset, str]):
        operationalstatus (Union[Unset, GetAllChargepointsOperationalstatus]):  Default:
            GetAllChargepointsOperationalstatus.AVAILABLE.
        accesstype (Union[Unset, GetAllChargepointsAccesstype]):  Default:
            GetAllChargepointsAccesstype.NEW.
        chargerpowertype (Union[Unset, GetAllChargepointsChargerpowertype]):  Default:
            GetAllChargepointsChargerpowertype.AC.
        organizationunitcode (Union[Unset, str]):
        inactiveonly (Union[Unset, bool]):
        firmwareversion (Union[Unset, str]):
        model (Union[Unset, str]):
        vendor (Union[Unset, str]):
        order_by (Union[Unset, GetAllChargepointsOrderBy]):  Default:
            GetAllChargepointsOrderBy.NAME.
        descending (Union[Unset, bool]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[List['ChargepointDto'], LongshipError]
    """

    return (
        await asyncio_detailed(
            client=client,
            skip=skip,
            take=take,
            search=search,
            is_online=is_online,
            tariff_id=tariff_id,
            operationalstatus=operationalstatus,
            accesstype=accesstype,
            chargerpowertype=chargerpowertype,
            organizationunitcode=organizationunitcode,
            inactiveonly=inactiveonly,
            firmwareversion=firmwareversion,
            model=model,
            vendor=vendor,
            order_by=order_by,
            descending=descending,
        )
    ).parsed



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union


T = TypeVar("T", bound="PriceInfoDto")


@_attrs_define
class PriceInfoDto:
    """
    Attributes:
        start_price (Union[Unset, float]):
        energy_price (Union[Unset, float]):
        total_parking_time_in_minutes (Union[Unset, float]):
        total_parking_time_steps (Union[Unset, int]):
        parking_time_price (Union[Unset, float]):
        total_charging_time_in_minutes (Union[Unset, float]):
        total_charging_time_steps (Union[Unset, int]):
        charging_time_price (Union[Unset, float]):
        total_price (Union[Unset, float]):
    """

    start_price: Union[Unset, float] = UNSET
    energy_price: Union[Unset, float] = UNSET
    total_parking_time_in_minutes: Union[Unset, float] = UNSET
    total_parking_time_steps: Union[Unset, int] = UNSET
    parking_time_price: Union[Unset, float] = UNSET
    total_charging_time_in_minutes: Union[Unset, float] = UNSET
    total_charging_time_steps: Union[Unset, int] = UNSET
    charging_time_price: Union[Unset, float] = UNSET
    total_price: Union[Unset, float] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        start_price = self.start_price

        energy_price = self.energy_price

        total_parking_time_in_minutes = self.total_parking_time_in_minutes

        total_parking_time_steps = self.total_parking_time_steps

        parking_time_price = self.parking_time_price

        total_charging_time_in_minutes = self.total_charging_time_in_minutes

        total_charging_time_steps = self.total_charging_time_steps

        charging_time_price = self.charging_time_price

        total_price = self.total_price

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if start_price is not UNSET:
            field_dict["startPrice"] = start_price
        if energy_price is not UNSET:
            field_dict["energyPrice"] = energy_price
        if total_parking_time_in_minutes is not UNSET:
            field_dict["totalParkingTimeInMinutes"] = total_parking_time_in_minutes
        if total_parking_time_steps is not UNSET:
            field_dict["totalParkingTimeSteps"] = total_parking_time_steps
        if parking_time_price is not UNSET:
            field_dict["parkingTimePrice"] = parking_time_price
        if total_charging_time_in_minutes is not UNSET:
            field_dict["totalChargingTimeInMinutes"] = total_charging_time_in_minutes
        if total_charging_time_steps is not UNSET:
            field_dict["totalChargingTimeSteps"] = total_charging_time_steps
        if charging_time_price is not UNSET:
            field_dict["chargingTimePrice"] = charging_time_price
        if total_price is not UNSET:
            field_dict["totalPrice"] = total_price

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        start_price = d.pop("startPrice", UNSET)

        energy_price = d.pop("energyPrice", UNSET)

        total_parking_time_in_minutes = d.pop("totalParkingTimeInMinutes", UNSET)

        total_parking_time_steps = d.pop("totalParkingTimeSteps", UNSET)

        parking_time_price = d.pop("parkingTimePrice", UNSET)

        total_charging_time_in_minutes = d.pop("totalChargingTimeInMinutes", UNSET)

        total_charging_time_steps = d.pop("totalChargingTimeSteps", UNSET)

        charging_time_price = d.pop("chargingTimePrice", UNSET)

        total_price = d.pop("totalPrice", UNSET)

        price_info_dto = cls(
            start_price=start_price,
            energy_price=energy_price,
            total_parking_time_in_minutes=total_parking_time_in_minutes,
            total_parking_time_steps=total_parking_time_steps,
            parking_time_price=parking_time_price,
            total_charging_time_in_minutes=total_charging_time_in_minutes,
            total_charging_time_steps=total_charging_time_steps,
            charging_time_price=charging_time_price,
            total_price=total_price,
        )

        price_info_dto.additional_properties = d
        return price_info_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union


T = TypeVar("T", bound="ChargepointPutDto")


@_attrs_define
class ChargepointPutDto:
    """
    Attributes:
        display_name (Union[Unset, str]):
        roaming_name (Union[Unset, str]):
        allow_any_token (Union[Unset, bool]):
        ou_code (Union[Unset, str]):
        tariff_id (Union[Unset, str]):
        sim_card_number (Union[Unset, str]):
        is_new (Union[Unset, bool]):
        max_capacity_in_kw (Union[Unset, float]):
    """

    display_name: Union[Unset, str] = UNSET
    roaming_name: Union[Unset, str] = UNSET
    allow_any_token: Union[Unset, bool] = UNSET
    ou_code: Union[Unset, str] = UNSET
    tariff_id: Union[Unset, str] = UNSET
    sim_card_number: Union[Unset, str] = UNSET
    is_new: Union[Unset, bool] = UNSET
    max_capacity_in_kw: Union[Unset, float] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        display_name = self.display_name

        roaming_name = self.roaming_name

        allow_any_token = self.allow_any_token

        ou_code = self.ou_code

        tariff_id = self.tariff_id

        sim_card_number = self.sim_card_number

        is_new = self.is_new

        max_capacity_in_kw = self.max_capacity_in_kw

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if display_name is not UNSET:
            field_dict["displayName"] = display_name
        if roaming_name is not UNSET:
            field_dict["roamingName"] = roaming_name
        if allow_any_token is not UNSET:
            field_dict["allowAnyToken"] = allow_any_token
        if ou_code is not UNSET:
            field_dict["ouCode"] = ou_code
        if tariff_id is not UNSET:
            field_dict["tariffId"] = tariff_id
        if sim_card_number is not UNSET:
            field_dict["simCardNumber"] = sim_card_number
        if is_new is not UNSET:
            field_dict["isNew"] = is_new
        if max_capacity_in_kw is not UNSET:
            field_dict["maxCapacityInKw"] = max_capacity_in_kw

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        display_name = d.pop("displayName", UNSET)

        roaming_name = d.pop("roamingName", UNSET)

        allow_any_token = d.pop("allowAnyToken", UNSET)

        ou_code = d.pop("ouCode", UNSET)

        tariff_id = d.pop("tariffId", UNSET)

        sim_card_number = d.pop("simCardNumber", UNSET)

        is_new = d.pop("isNew", UNSET)

        max_capacity_in_kw = d.pop("maxCapacityInKw", UNSET)

        chargepoint_put_dto = cls(
            display_name=display_name,
            roaming_name=roaming_name,
            allow_any_token=allow_any_token,
            ou_code=ou_code,
            tariff_id=tariff_id,
            sim_card_number=sim_card_number,
            is_new=is_new,
            max_capacity_in_kw=max_capacity_in_kw,
        )

        chargepoint_put_dto.additional_properties = d
        return chargepoint_put_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
from ..models.environmental_impact_dto_category import EnvironmentalImpactDtoCategory


T = TypeVar("T", bound="EnvironmentalImpactDto")


@_attrs_define
class EnvironmentalImpactDto:
    """
    Attributes:
        category (Union[Unset, EnvironmentalImpactDtoCategory]):  Default: EnvironmentalImpactDtoCategory.NUCLEAR_WASTE.
        amount (Union[Unset, int]):
    """

    category: Union[Unset, EnvironmentalImpactDtoCategory] = (
        EnvironmentalImpactDtoCategory.NUCLEAR_WASTE
    )
    amount: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        category: Union[Unset, str] = UNSET
        if not isinstance(self.category, Unset):
            category = self.category.value

        amount = self.amount

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if category is not UNSET:
            field_dict["category"] = category
        if amount is not UNSET:
            field_dict["amount"] = amount

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        _category = d.pop("category", UNSET)
        category: Union[Unset, EnvironmentalImpactDtoCategory]
        if isinstance(_category, Unset) or _category is None:
            category = UNSET
        else:
            category = EnvironmentalImpactDtoCategory(_category)

        amount = d.pop("amount", UNSET)

        environmental_impact_dto = cls(
            category=category,
            amount=amount,
        )

        environmental_impact_dto.additional_properties = d
        return environmental_impact_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class TriggerMessageRequestRequestedMessage(str, Enum):
    BOOTNOTIFICATION = "BootNotification"
    DIAGNOSTICSSTATUSNOTIFICATION = "DiagnosticsStatusNotification"
    FIRMWARESTATUSNOTIFICATION = "FirmwareStatusNotification"
    HEARTBEAT = "Heartbeat"
    METERVALUES = "MeterValues"
    STATUSNOTIFICATION = "StatusNotification"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class GetAllChargepointsOperationalstatus(str, Enum):
    AVAILABLE = "available"
    CHARGING = "charging"
    FAULTED = "faulted"
    FINISHING = "finishing"
    PREPARING = "preparing"
    RESERVED = "reserved"
    SUSPENDEDEV = "suspendedEV"
    SUSPENDEDEVSE = "suspendedEVSE"
    UNAVAILABLE = "unavailable"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union


T = TypeVar("T", bound="TariffDistributionPutDto")


@_attrs_define
class TariffDistributionPutDto:
    """
    Attributes:
        name (Union[Unset, str]):
        ou_code (Union[Unset, str]):
        energy_compensation (Union[Unset, float]):
        fixed_tenant_k_wh_fee (Union[Unset, float]):
        percentage_fee_customer (Union[Unset, float]):
        percentage_fee_tenant (Union[Unset, float]):
    """

    name: Union[Unset, str] = UNSET
    ou_code: Union[Unset, str] = UNSET
    energy_compensation: Union[Unset, float] = UNSET
    fixed_tenant_k_wh_fee: Union[Unset, float] = UNSET
    percentage_fee_customer: Union[Unset, float] = UNSET
    percentage_fee_tenant: Union[Unset, float] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        ou_code = self.ou_code

        energy_compensation = self.energy_compensation

        fixed_tenant_k_wh_fee = self.fixed_tenant_k_wh_fee

        percentage_fee_customer = self.percentage_fee_customer

        percentage_fee_tenant = self.percentage_fee_tenant

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if ou_code is not UNSET:
            field_dict["ouCode"] = ou_code
        if energy_compensation is not UNSET:
            field_dict["energyCompensation"] = energy_compensation
        if fixed_tenant_k_wh_fee is not UNSET:
            field_dict["fixedTenantKWhFee"] = fixed_tenant_k_wh_fee
        if percentage_fee_customer is not UNSET:
            field_dict["percentageFeeCustomer"] = percentage_fee_customer
        if percentage_fee_tenant is not UNSET:
            field_dict["percentageFeeTenant"] = percentage_fee_tenant

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name", UNSET)

        ou_code = d.pop("ouCode", UNSET)

        energy_compensation = d.pop("energyCompensation", UNSET)

        fixed_tenant_k_wh_fee = d.pop("fixedTenantKWhFee", UNSET)

        percentage_fee_customer = d.pop("percentageFeeCustomer", UNSET)

        percentage_fee_tenant = d.pop("percentageFeeTenant", UNSET)

        tariff_distribution_put_dto = cls(
            name=name,
            ou_code=ou_code,
            energy_compensation=energy_compensation,
            fixed_tenant_k_wh_fee=fixed_tenant_k_wh_fee,
            percentage_fee_customer=percentage_fee_customer,
            percentage_fee_tenant=percentage_fee_tenant,
        )

        tariff_distribution_put_dto.additional_properties = d
        return tariff_distribution_put_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ConnectorOperationalStatusDtoOperationalStatus(str, Enum):
    AVAILABLE = "Available"
    CHARGING = "Charging"
    FAULTED = "Faulted"
    FINISHING = "Finishing"
    PREPARING = "Preparing"
    RESERVED = "Reserved"
    SUSPENDEDEV = "SuspendedEV"
    SUSPENDEDEVSE = "SuspendedEVSE"
    UNAVAILABLE = "Unavailable"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class LocationEVSEDtoCapabilitiesItem(str, Enum):
    CHARGING_PREFERENCES_CAPABLE = "CHARGING_PREFERENCES_CAPABLE"
    CHARGING_PROFILE_CAPABLE = "CHARGING_PROFILE_CAPABLE"
    CHIP_CARD_SUPPORT = "CHIP_CARD_SUPPORT"
    CONTACTLESS_CARD_SUPPORT = "CONTACTLESS_CARD_SUPPORT"
    CREDIT_CARD_PAYABLE = "CREDIT_CARD_PAYABLE"
    DEBIT_CARD_PAYABLE = "DEBIT_CARD_PAYABLE"
    PED_TERMINAL = "PED_TERMINAL"
    REMOTE_START_STOP_CAPABLE = "REMOTE_START_STOP_CAPABLE"
    RESERVABLE = "RESERVABLE"
    RFID_READER = "RFID_READER"
    TOKEN_GROUP_CAPABLE = "TOKEN_GROUP_CAPABLE"
    UNLOCK_CAPABLE = "UNLOCK_CAPABLE"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.chargepoint_connector_dto_format import ChargepointConnectorDtoFormat
from ..models.chargepoint_connector_dto_standard import ChargepointConnectorDtoStandard
from ..models.chargepoint_connector_dto_power_type import (
    ChargepointConnectorDtoPowerType,
)
from typing import Union
from ..models.chargepoint_connector_dto_operational_status import (
    ChargepointConnectorDtoOperationalStatus,
)


T = TypeVar("T", bound="ChargepointConnectorDto")


@_attrs_define
class ChargepointConnectorDto:
    """
    Attributes:
        id (Union[Unset, str]):
        operational_status (Union[Unset, ChargepointConnectorDtoOperationalStatus]):  Default:
            ChargepointConnectorDtoOperationalStatus.AVAILABLE.
        standard (Union[Unset, ChargepointConnectorDtoStandard]):  Default: ChargepointConnectorDtoStandard.CHADEMO.
        format_ (Union[Unset, ChargepointConnectorDtoFormat]):  Default: ChargepointConnectorDtoFormat.SOCKET.
        power_type (Union[Unset, ChargepointConnectorDtoPowerType]):  Default:
            ChargepointConnectorDtoPowerType.AC_1_PHASE.
        max_voltage (Union[Unset, int]):
        max_amperage (Union[Unset, int]):
        max_electric_power (Union[Unset, int]):
        tariff_id (Union[Unset, str]):
    """

    id: Union[Unset, str] = UNSET
    operational_status: Union[Unset, ChargepointConnectorDtoOperationalStatus] = (
        ChargepointConnectorDtoOperationalStatus.AVAILABLE
    )
    standard: Union[Unset, ChargepointConnectorDtoStandard] = (
        ChargepointConnectorDtoStandard.CHADEMO
    )
    format_: Union[Unset, ChargepointConnectorDtoFormat] = (
        ChargepointConnectorDtoFormat.SOCKET
    )
    power_type: Union[Unset, ChargepointConnectorDtoPowerType] = (
        ChargepointConnectorDtoPowerType.AC_1_PHASE
    )
    max_voltage: Union[Unset, int] = UNSET
    max_amperage: Union[Unset, int] = UNSET
    max_electric_power: Union[Unset, int] = UNSET
    tariff_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        operational_status: Union[Unset, str] = UNSET
        if not isinstance(self.operational_status, Unset):
            operational_status = self.operational_status.value

        standard: Union[Unset, str] = UNSET
        if not isinstance(self.standard, Unset):
            standard = self.standard.value

        format_: Union[Unset, str] = UNSET
        if not isinstance(self.format_, Unset):
            format_ = self.format_.value

        power_type: Union[Unset, str] = UNSET
        if not isinstance(self.power_type, Unset):
            power_type = self.power_type.value

        max_voltage = self.max_voltage

        max_amperage = self.max_amperage

        max_electric_power = self.max_electric_power

        tariff_id = self.tariff_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if operational_status is not UNSET:
            field_dict["operationalStatus"] = operational_status
        if standard is not UNSET:
            field_dict["standard"] = standard
        if format_ is not UNSET:
            field_dict["format"] = format_
        if power_type is not UNSET:
            field_dict["powerType"] = power_type
        if max_voltage is not UNSET:
            field_dict["maxVoltage"] = max_voltage
        if max_amperage is not UNSET:
            field_dict["maxAmperage"] = max_amperage
        if max_electric_power is not UNSET:
            field_dict["maxElectricPower"] = max_electric_power
        if tariff_id is not UNSET:
            field_dict["tariffId"] = tariff_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        _operational_status = d.pop("operationalStatus", UNSET)
        operational_status: Union[Unset, ChargepointConnectorDtoOperationalStatus]
        if isinstance(_operational_status, Unset) or _operational_status is None:
            operational_status = UNSET
        else:
            operational_status = ChargepointConnectorDtoOperationalStatus(
                _operational_status
            )

        _standard = d.pop("standard", UNSET)
        standard: Union[Unset, ChargepointConnectorDtoStandard]
        if isinstance(_standard, Unset) or _standard is None:
            standard = UNSET
        else:
            standard = ChargepointConnectorDtoStandard(_standard)

        _format_ = d.pop("format", UNSET)
        format_: Union[Unset, ChargepointConnectorDtoFormat]
        if isinstance(_format_, Unset) or _format_ is None:
            format_ = UNSET
        else:
            format_ = ChargepointConnectorDtoFormat(_format_)

        _power_type = d.pop("powerType", UNSET)
        power_type: Union[Unset, ChargepointConnectorDtoPowerType]
        if isinstance(_power_type, Unset) or _power_type is None:
            power_type = UNSET
        else:
            power_type = ChargepointConnectorDtoPowerType(_power_type)

        max_voltage = d.pop("maxVoltage", UNSET)

        max_amperage = d.pop("maxAmperage", UNSET)

        max_electric_power = d.pop("maxElectricPower", UNSET)

        tariff_id = d.pop("tariffId", UNSET)

        chargepoint_connector_dto = cls(
            id=id,
            operational_status=operational_status,
            standard=standard,
            format_=format_,
            power_type=power_type,
            max_voltage=max_voltage,
            max_amperage=max_amperage,
            max_electric_power=max_electric_power,
            tariff_id=tariff_id,
        )

        chargepoint_connector_dto.additional_properties = d
        return chargepoint_connector_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.reimburse_started_by_info_dto_authorization_state import (
    ReimburseStartedByInfoDtoAuthorizationState,
)
from typing import Union

if TYPE_CHECKING:
    from ..models.reimburse_started_by_token_dto import ReimburseStartedByTokenDto


T = TypeVar("T", bound="ReimburseStartedByInfoDto")


@_attrs_define
class ReimburseStartedByInfoDto:
    """
    Attributes:
        id_tag (Union[Unset, str]):
        token_info (Union[Unset, ReimburseStartedByTokenDto]):
        authorization_state (Union[Unset, ReimburseStartedByInfoDtoAuthorizationState]):  Default:
            ReimburseStartedByInfoDtoAuthorizationState.APPROVEDBYREMOTE.
        is_guest_usage (Union[Unset, bool]):
    """

    id_tag: Union[Unset, str] = UNSET
    token_info: Union[Unset, "ReimburseStartedByTokenDto"] = UNSET
    authorization_state: Union[Unset, ReimburseStartedByInfoDtoAuthorizationState] = (
        ReimburseStartedByInfoDtoAuthorizationState.APPROVEDBYREMOTE
    )
    is_guest_usage: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id_tag = self.id_tag

        token_info: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.token_info, Unset):
            token_info = self.token_info.to_dict()

        authorization_state: Union[Unset, str] = UNSET
        if not isinstance(self.authorization_state, Unset):
            authorization_state = self.authorization_state.value

        is_guest_usage = self.is_guest_usage

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id_tag is not UNSET:
            field_dict["idTag"] = id_tag
        if token_info is not UNSET:
            field_dict["tokenInfo"] = token_info
        if authorization_state is not UNSET:
            field_dict["authorizationState"] = authorization_state
        if is_guest_usage is not UNSET:
            field_dict["isGuestUsage"] = is_guest_usage

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.reimburse_started_by_token_dto import ReimburseStartedByTokenDto

        d = src_dict.copy()
        id_tag = d.pop("idTag", UNSET)

        _token_info = d.pop("tokenInfo", UNSET)
        token_info: Union[Unset, ReimburseStartedByTokenDto]
        if isinstance(_token_info, Unset) or _token_info is None:
            token_info = UNSET
        else:
            token_info = ReimburseStartedByTokenDto.from_dict(_token_info)

        _authorization_state = d.pop("authorizationState", UNSET)
        authorization_state: Union[Unset, ReimburseStartedByInfoDtoAuthorizationState]
        if isinstance(_authorization_state, Unset) or _authorization_state is None:
            authorization_state = UNSET
        else:
            authorization_state = ReimburseStartedByInfoDtoAuthorizationState(
                _authorization_state
            )

        is_guest_usage = d.pop("isGuestUsage", UNSET)

        reimburse_started_by_info_dto = cls(
            id_tag=id_tag,
            token_info=token_info,
            authorization_state=authorization_state,
            is_guest_usage=is_guest_usage,
        )

        reimburse_started_by_info_dto.additional_properties = d
        return reimburse_started_by_info_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ChargingMeterValueDtoMeasurand(str, Enum):
    CURRENT_EXPORT = "Current.Export"
    CURRENT_IMPORT = "Current.Import"
    CURRENT_OFFERED = "Current.Offered"
    ENERGY_ACTIVE_EXPORT_INTERVAL = "Energy.Active.Export.Interval"
    ENERGY_ACTIVE_EXPORT_REGISTER = "Energy.Active.Export.Register"
    ENERGY_ACTIVE_IMPORT_INTERVAL = "Energy.Active.Import.Interval"
    ENERGY_ACTIVE_IMPORT_REGISTER = "Energy.Active.Import.Register"
    ENERGY_REACTIVE_EXPORT_INTERVAL = "Energy.Reactive.Export.Interval"
    ENERGY_REACTIVE_EXPORT_REGISTER = "Energy.Reactive.Export.Register"
    ENERGY_REACTIVE_IMPORT_INTERVAL = "Energy.Reactive.Import.Interval"
    ENERGY_REACTIVE_IMPORT_REGISTER = "Energy.Reactive.Import.Register"
    FREQUENCY = "Frequency"
    POWER_ACTIVE_EXPORT = "Power.Active.Export"
    POWER_ACTIVE_IMPORT = "Power.Active.Import"
    POWER_FACTOR = "Power.Factor"
    POWER_OFFERED = "Power.Offered"
    POWER_REACTIVE_EXPORT = "Power.Reactive.Export"
    POWER_REACTIVE_IMPORT = "Power.Reactive.Import"
    RPM = "RPM"
    SOC = "SoC"
    TEMPERATURE = "Temperature"
    VOLTAGE = "Voltage"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union
from ..models.image_dto_category import ImageDtoCategory


T = TypeVar("T", bound="ImageDto")


@_attrs_define
class ImageDto:
    """
    Attributes:
        url (Union[Unset, str]):
        thumbnail (Union[Unset, str]):
        category (Union[Unset, ImageDtoCategory]):  Default: ImageDtoCategory.CHARGER.
        type (Union[Unset, str]):
        width (Union[Unset, int]):
        height (Union[Unset, int]):
    """

    url: Union[Unset, str] = UNSET
    thumbnail: Union[Unset, str] = UNSET
    category: Union[Unset, ImageDtoCategory] = ImageDtoCategory.CHARGER
    type: Union[Unset, str] = UNSET
    width: Union[Unset, int] = UNSET
    height: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        url = self.url

        thumbnail = self.thumbnail

        category: Union[Unset, str] = UNSET
        if not isinstance(self.category, Unset):
            category = self.category.value

        type = self.type

        width = self.width

        height = self.height

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if url is not UNSET:
            field_dict["url"] = url
        if thumbnail is not UNSET:
            field_dict["thumbnail"] = thumbnail
        if category is not UNSET:
            field_dict["category"] = category
        if type is not UNSET:
            field_dict["type"] = type
        if width is not UNSET:
            field_dict["width"] = width
        if height is not UNSET:
            field_dict["height"] = height

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        if src_dict is None:
            return UNSET
        d = src_dict.copy()
        url = d.pop("url", UNSET)

        thumbnail = d.pop("thumbnail", UNSET)

        _category = d.pop("category", UNSET)
        category: Union[Unset, ImageDtoCategory]
        if isinstance(_category, Unset) or _category is None:
            category = UNSET
        else:
            category = ImageDtoCategory(_category)

        type = d.pop("type", UNSET)

        width = d.pop("width", UNSET)

        height = d.pop("height", UNSET)

        image_dto = cls(
            url=url,
            thumbnail=thumbnail,
            category=category,
            type=type,
            width=width,
            height=height,
        )

        image_dto.additional_properties = d
        return image_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from dateutil.parser import isoparse
import datetime
from ..models.id_tag_info_status import IdTagInfoStatus
from typing import Union


T = TypeVar("T", bound="IdTagInfo")


@_attrs_define
class IdTagInfo:
    """
    Attributes:
        status (IdTagInfoStatus):  Default: IdTagInfoStatus.ACCEPTED.
        expiry_date (Union[Unset, datetime.datetime]):
        parent_id_tag (Union[Unset, str]):
    """

    status: IdTagInfoStatus = IdTagInfoStatus.ACCEPTED
    expiry_date: Union[Unset, datetime.datetime] = UNSET
    parent_id_tag: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        status = self.status.value

        expiry_date: Union[Unset, str] = UNSET
        if not isinstance(self.expiry_date, Unset):
            expiry_date = self.expiry_date.isoformat()

        parent_id_tag = self.parent_id_tag

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "status": status,
            }
        )
        if expiry_date is not UNSET:
            field_dict["expiryDate"] = expiry_date
        if parent_id_tag is not UNSET:
            field_dict["parentIdTag"] = parent_id_tag

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        status = IdTagInfoStatus(d.pop("status"))

        _expiry_date = d.pop("expiryDate", UNSET)
        expiry_date: Union[Unset, datetime.datetime]
        if isinstance(_expiry_date, Unset) or _expiry_date is None:
            expiry_date = UNSET
        else:
            expiry_date = isoparse(_expiry_date)

        parent_id_tag = d.pop("parentIdTag", UNSET)

        id_tag_info = cls(
            status=status,
            expiry_date=expiry_date,
            parent_id_tag=parent_id_tag,
        )

        id_tag_info.additional_properties = d
        return id_tag_info

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ChargepointConnectorDtoStandard(str, Enum):
    CHADEMO = "CHADEMO"
    DOMESTIC_A = "DOMESTIC_A"
    DOMESTIC_B = "DOMESTIC_B"
    DOMESTIC_C = "DOMESTIC_C"
    DOMESTIC_D = "DOMESTIC_D"
    DOMESTIC_E = "DOMESTIC_E"
    DOMESTIC_F = "DOMESTIC_F"
    DOMESTIC_G = "DOMESTIC_G"
    DOMESTIC_H = "DOMESTIC_H"
    DOMESTIC_I = "DOMESTIC_I"
    DOMESTIC_J = "DOMESTIC_J"
    DOMESTIC_K = "DOMESTIC_K"
    DOMESTIC_L = "DOMESTIC_L"
    IEC_60309_2_SINGLE_16 = "IEC_60309_2_single_16"
    IEC_60309_2_THREE_16 = "IEC_60309_2_three_16"
    IEC_60309_2_THREE_32 = "IEC_60309_2_three_32"
    IEC_60309_2_THREE_64 = "IEC_60309_2_three_64"
    IEC_62196_T1 = "IEC_62196_T1"
    IEC_62196_T1_COMBO = "IEC_62196_T1_COMBO"
    IEC_62196_T2 = "IEC_62196_T2"
    IEC_62196_T2_COMBO = "IEC_62196_T2_COMBO"
    IEC_62196_T3A = "IEC_62196_T3A"
    IEC_62196_T3C = "IEC_62196_T3C"
    PANTOGRAPH_BOTTOM_UP = "PANTOGRAPH_BOTTOM_UP"
    PANTOGRAPH_TOP_DOWN = "PANTOGRAPH_TOP_DOWN"
    TESLA_R = "TESLA_R"
    TESLA_S = "TESLA_S"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class ReimburseInfoDtoType(str, Enum):
    NONE = "NONE"
    ORGANIZATIONUNIT = "ORGANIZATIONUNIT"
    TOKEN = "TOKEN"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union

if TYPE_CHECKING:
    from ..models.image_dto import ImageDto


T = TypeVar("T", bound="BusinessDetailsDto")


@_attrs_define
class BusinessDetailsDto:
    """
    Attributes:
        name (Union[Unset, str]):
        website (Union[Unset, str]):
        image (Union[Unset, ImageDto]):
    """

    name: Union[Unset, str] = UNSET
    website: Union[Unset, str] = UNSET
    image: Union[Unset, "ImageDto"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        website = self.website

        image: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.image, Unset):
            image = self.image.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if website is not UNSET:
            field_dict["website"] = website
        if image is not UNSET:
            field_dict["image"] = image

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.image_dto import ImageDto

        d = src_dict.copy()
        name = d.pop("name", UNSET)

        website = d.pop("website", UNSET)

        _image = d.pop("image", UNSET)
        image: Union[Unset, ImageDto]
        if isinstance(_image, Unset) or _image is None:
            image = UNSET
        else:
            image = ImageDto.from_dict(_image)

        business_details_dto = cls(
            name=name,
            website=website,
            image=image,
        )

        business_details_dto.additional_properties = d
        return business_details_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union

if TYPE_CHECKING:
    from ..models.chargepoint_connector_dto import ChargepointConnectorDto


T = TypeVar("T", bound="ChargepointEVSEDto")


@_attrs_define
class ChargepointEVSEDto:
    """
    Attributes:
        evse_id (Union[Unset, str]):
        connectors (Union[Unset, List['ChargepointConnectorDto']]):
    """

    evse_id: Union[Unset, str] = UNSET
    connectors: Union[Unset, List["ChargepointConnectorDto"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        evse_id = self.evse_id

        connectors: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.connectors, Unset):
            connectors = []
            for connectors_item_data in self.connectors:
                connectors_item = connectors_item_data.to_dict()
                connectors.append(connectors_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if evse_id is not UNSET:
            field_dict["evse_id"] = evse_id
        if connectors is not UNSET:
            field_dict["connectors"] = connectors

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.chargepoint_connector_dto import ChargepointConnectorDto

        d = src_dict.copy()
        evse_id = d.pop("evse_id", UNSET)

        connectors = []
        _connectors = d.pop("connectors", UNSET)
        for connectors_item_data in _connectors or []:
            connectors_item = ChargepointConnectorDto.from_dict(connectors_item_data)

            connectors.append(connectors_item)

        chargepoint_evse_dto = cls(
            evse_id=evse_id,
            connectors=connectors,
        )

        chargepoint_evse_dto.additional_properties = d
        return chargepoint_evse_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union

if TYPE_CHECKING:
    from ..models.charging_profile import ChargingProfile


T = TypeVar("T", bound="RemoteStartTransactionRequest")


@_attrs_define
class RemoteStartTransactionRequest:
    """
    Attributes:
        id_tag (str):
        connector_id (Union[Unset, int]):
        charging_profile (Union[Unset, ChargingProfile]):
    """

    id_tag: str
    connector_id: Union[Unset, int] = UNSET
    charging_profile: Union[Unset, "ChargingProfile"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id_tag = self.id_tag

        connector_id = self.connector_id

        charging_profile: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.charging_profile, Unset):
            charging_profile = self.charging_profile.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "idTag": id_tag,
            }
        )
        if connector_id is not UNSET:
            field_dict["connectorId"] = connector_id
        if charging_profile is not UNSET:
            field_dict["chargingProfile"] = charging_profile

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.charging_profile import ChargingProfile

        d = src_dict.copy()
        id_tag = d.pop("idTag")

        connector_id = d.pop("connectorId", UNSET)

        _charging_profile = d.pop("chargingProfile", UNSET)
        charging_profile: Union[Unset, ChargingProfile]
        if isinstance(_charging_profile, Unset) or _charging_profile is None:
            charging_profile = UNSET
        else:
            charging_profile = ChargingProfile.from_dict(_charging_profile)

        remote_start_transaction_request = cls(
            id_tag=id_tag,
            connector_id=connector_id,
            charging_profile=charging_profile,
        )

        remote_start_transaction_request.additional_properties = d
        return remote_start_transaction_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.cdr_started_by_info_dto_roaming_platform_type import (
    CdrStartedByInfoDtoRoamingPlatformType,
)
from typing import Union
from ..models.cdr_started_by_info_dto_authorization_state import (
    CdrStartedByInfoDtoAuthorizationState,
)

if TYPE_CHECKING:
    from ..models.cdr_started_by_token_dto import CdrStartedByTokenDto


T = TypeVar("T", bound="CdrStartedByInfoDto")


@_attrs_define
class CdrStartedByInfoDto:
    """
    Attributes:
        token_info (Union[Unset, CdrStartedByTokenDto]):
        roaming_platform_type (Union[Unset, CdrStartedByInfoDtoRoamingPlatformType]):  Default:
            CdrStartedByInfoDtoRoamingPlatformType.HUBJECT.
        authorization_state (Union[Unset, CdrStartedByInfoDtoAuthorizationState]):  Default:
            CdrStartedByInfoDtoAuthorizationState.APPROVEDBYREMOTE.
        roaming_platform_connection_id (Union[Unset, str]):
        is_guest_usage (Union[Unset, bool]):
    """

    token_info: Union[Unset, "CdrStartedByTokenDto"] = UNSET
    roaming_platform_type: Union[Unset, CdrStartedByInfoDtoRoamingPlatformType] = (
        CdrStartedByInfoDtoRoamingPlatformType.HUBJECT
    )
    authorization_state: Union[Unset, CdrStartedByInfoDtoAuthorizationState] = (
        CdrStartedByInfoDtoAuthorizationState.APPROVEDBYREMOTE
    )
    roaming_platform_connection_id: Union[Unset, str] = UNSET
    is_guest_usage: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        token_info: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.token_info, Unset):
            token_info = self.token_info.to_dict()

        roaming_platform_type: Union[Unset, str] = UNSET
        if not isinstance(self.roaming_platform_type, Unset):
            roaming_platform_type = self.roaming_platform_type.value

        authorization_state: Union[Unset, str] = UNSET
        if not isinstance(self.authorization_state, Unset):
            authorization_state = self.authorization_state.value

        roaming_platform_connection_id = self.roaming_platform_connection_id

        is_guest_usage = self.is_guest_usage

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if token_info is not UNSET:
            field_dict["tokenInfo"] = token_info
        if roaming_platform_type is not UNSET:
            field_dict["roamingPlatformType"] = roaming_platform_type
        if authorization_state is not UNSET:
            field_dict["authorizationState"] = authorization_state
        if roaming_platform_connection_id is not UNSET:
            field_dict["roamingPlatformConnectionId"] = roaming_platform_connection_id
        if is_guest_usage is not UNSET:
            field_dict["isGuestUsage"] = is_guest_usage

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.cdr_started_by_token_dto import CdrStartedByTokenDto

        d = src_dict.copy()
        _token_info = d.pop("tokenInfo", UNSET)
        token_info: Union[Unset, CdrStartedByTokenDto]
        if isinstance(_token_info, Unset) or _token_info is None:
            token_info = UNSET
        else:
            token_info = CdrStartedByTokenDto.from_dict(_token_info)

        _roaming_platform_type = d.pop("roamingPlatformType", UNSET)
        roaming_platform_type: Union[Unset, CdrStartedByInfoDtoRoamingPlatformType]
        if isinstance(_roaming_platform_type, Unset) or _roaming_platform_type is None:
            roaming_platform_type = UNSET
        else:
            roaming_platform_type = CdrStartedByInfoDtoRoamingPlatformType(
                _roaming_platform_type
            )

        _authorization_state = d.pop("authorizationState", UNSET)
        authorization_state: Union[Unset, CdrStartedByInfoDtoAuthorizationState]
        if isinstance(_authorization_state, Unset) or _authorization_state is None:
            authorization_state = UNSET
        else:
            authorization_state = CdrStartedByInfoDtoAuthorizationState(
                _authorization_state
            )

        roaming_platform_connection_id = d.pop("roamingPlatformConnectionId", UNSET)

        is_guest_usage = d.pop("isGuestUsage", UNSET)

        cdr_started_by_info_dto = cls(
            token_info=token_info,
            roaming_platform_type=roaming_platform_type,
            authorization_state=authorization_state,
            roaming_platform_connection_id=roaming_platform_connection_id,
            is_guest_usage=is_guest_usage,
        )

        cdr_started_by_info_dto.additional_properties = d
        return cdr_started_by_info_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union


T = TypeVar("T", bound="SessionGeoLocationDto")


@_attrs_define
class SessionGeoLocationDto:
    """
    Attributes:
        latitude (Union[Unset, str]):
        longitude (Union[Unset, str]):
    """

    latitude: Union[Unset, str] = UNSET
    longitude: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        latitude = self.latitude

        longitude = self.longitude

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if latitude is not UNSET:
            field_dict["latitude"] = latitude
        if longitude is not UNSET:
            field_dict["longitude"] = longitude

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        latitude = d.pop("latitude", UNSET)

        longitude = d.pop("longitude", UNSET)

        session_geo_location_dto = cls(
            latitude=latitude,
            longitude=longitude,
        )

        session_geo_location_dto.additional_properties = d
        return session_geo_location_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class SessionDtoStatus(str, Enum):
    ACTIVE = "ACTIVE"
    COMPLETED = "COMPLETED"
    INVALID = "INVALID"
    PENDING = "PENDING"
    RESERVATION = "RESERVATION"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from dateutil.parser import isoparse
import datetime
from typing import Union


T = TypeVar("T", bound="ReserveNowRequest")


@_attrs_define
class ReserveNowRequest:
    """
    Attributes:
        connector_id (int):
        expiry_date (datetime.datetime):
        id_tag (str):
        reservation_id (int):
        parent_id_tag (Union[Unset, str]):
    """

    connector_id: int
    expiry_date: datetime.datetime
    id_tag: str
    reservation_id: int
    parent_id_tag: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        connector_id = self.connector_id

        expiry_date = self.expiry_date.isoformat()

        id_tag = self.id_tag

        reservation_id = self.reservation_id

        parent_id_tag = self.parent_id_tag

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "connectorId": connector_id,
                "expiryDate": expiry_date,
                "idTag": id_tag,
                "reservationId": reservation_id,
            }
        )
        if parent_id_tag is not UNSET:
            field_dict["parentIdTag"] = parent_id_tag

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        connector_id = d.pop("connectorId")

        expiry_date = isoparse(d.pop("expiryDate"))

        id_tag = d.pop("idTag")

        reservation_id = d.pop("reservationId")

        parent_id_tag = d.pop("parentIdTag", UNSET)

        reserve_now_request = cls(
            connector_id=connector_id,
            expiry_date=expiry_date,
            id_tag=id_tag,
            reservation_id=reservation_id,
            parent_id_tag=parent_id_tag,
        )

        reserve_now_request.additional_properties = d
        return reserve_now_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from dateutil.parser import isoparse
from ..models.location_evse_dto_parking_restrictions_item import (
    LocationEVSEDtoParkingRestrictionsItem,
)
from ..models.location_evse_dto_capabilities_item import LocationEVSEDtoCapabilitiesItem
import datetime
from typing import Union
from ..models.location_evse_dto_status import LocationEVSEDtoStatus

if TYPE_CHECKING:
    from ..models.display_text_dto import DisplayTextDto
    from ..models.image_dto import ImageDto
    from ..models.geo_location_dto import GeoLocationDto
    from ..models.status_schedule_dto import StatusScheduleDto
    from ..models.connector_dto import ConnectorDto


T = TypeVar("T", bound="LocationEVSEDto")


@_attrs_define
class LocationEVSEDto:
    """
    Attributes:
        id (Union[Unset, str]):
        chargepointid (Union[Unset, str]):
        latitude (Union[Unset, float]):
        longitude (Union[Unset, float]):
        uid (Union[Unset, str]):
        evse_id (Union[Unset, str]):
        status (Union[Unset, LocationEVSEDtoStatus]):  Default: LocationEVSEDtoStatus.AVAILABLE.
        status_schedule (Union[Unset, List['StatusScheduleDto']]):
        capabilities (Union[Unset, List[LocationEVSEDtoCapabilitiesItem]]):
        connectors (Union[Unset, List['ConnectorDto']]):
        floor_level (Union[Unset, str]):
        coordinates (Union[Unset, GeoLocationDto]):
        physical_reference (Union[Unset, str]):
        directions (Union[Unset, List['DisplayTextDto']]):
        parking_restrictions (Union[Unset, List[LocationEVSEDtoParkingRestrictionsItem]]):
        images (Union[Unset, List['ImageDto']]):
        last_updated (Union[Unset, datetime.datetime]):
    """

    id: Union[Unset, str] = UNSET
    chargepointid: Union[Unset, str] = UNSET
    latitude: Union[Unset, float] = UNSET
    longitude: Union[Unset, float] = UNSET
    uid: Union[Unset, str] = UNSET
    evse_id: Union[Unset, str] = UNSET
    status: Union[Unset, LocationEVSEDtoStatus] = LocationEVSEDtoStatus.AVAILABLE
    status_schedule: Union[Unset, List["StatusScheduleDto"]] = UNSET
    capabilities: Union[Unset, List[LocationEVSEDtoCapabilitiesItem]] = UNSET
    connectors: Union[Unset, List["ConnectorDto"]] = UNSET
    floor_level: Union[Unset, str] = UNSET
    coordinates: Union[Unset, "GeoLocationDto"] = UNSET
    physical_reference: Union[Unset, str] = UNSET
    directions: Union[Unset, List["DisplayTextDto"]] = UNSET
    parking_restrictions: Union[Unset, List[LocationEVSEDtoParkingRestrictionsItem]] = (
        UNSET
    )
    images: Union[Unset, List["ImageDto"]] = UNSET
    last_updated: Union[Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        chargepointid = self.chargepointid

        latitude = self.latitude

        longitude = self.longitude

        uid = self.uid

        evse_id = self.evse_id

        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        status_schedule: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.status_schedule, Unset):
            status_schedule = []
            for status_schedule_item_data in self.status_schedule:
                status_schedule_item = status_schedule_item_data.to_dict()
                status_schedule.append(status_schedule_item)

        capabilities: Union[Unset, List[str]] = UNSET
        if not isinstance(self.capabilities, Unset):
            capabilities = []
            for capabilities_item_data in self.capabilities:
                capabilities_item = capabilities_item_data.value
                capabilities.append(capabilities_item)

        connectors: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.connectors, Unset):
            connectors = []
            for connectors_item_data in self.connectors:
                connectors_item = connectors_item_data.to_dict()
                connectors.append(connectors_item)

        floor_level = self.floor_level

        coordinates: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.coordinates, Unset):
            coordinates = self.coordinates.to_dict()

        physical_reference = self.physical_reference

        directions: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.directions, Unset):
            directions = []
            for directions_item_data in self.directions:
                directions_item = directions_item_data.to_dict()
                directions.append(directions_item)

        parking_restrictions: Union[Unset, List[str]] = UNSET
        if not isinstance(self.parking_restrictions, Unset):
            parking_restrictions = []
            for parking_restrictions_item_data in self.parking_restrictions:
                parking_restrictions_item = parking_restrictions_item_data.value
                parking_restrictions.append(parking_restrictions_item)

        images: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.images, Unset):
            images = []
            for images_item_data in self.images:
                images_item = images_item_data.to_dict()
                images.append(images_item)

        last_updated: Union[Unset, str] = UNSET
        if not isinstance(self.last_updated, Unset):
            last_updated = self.last_updated.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if chargepointid is not UNSET:
            field_dict["chargepointid"] = chargepointid
        if latitude is not UNSET:
            field_dict["latitude"] = latitude
        if longitude is not UNSET:
            field_dict["longitude"] = longitude
        if uid is not UNSET:
            field_dict["uid"] = uid
        if evse_id is not UNSET:
            field_dict["evse_id"] = evse_id
        if status is not UNSET:
            field_dict["status"] = status
        if status_schedule is not UNSET:
            field_dict["status_schedule"] = status_schedule
        if capabilities is not UNSET:
            field_dict["capabilities"] = capabilities
        if connectors is not UNSET:
            field_dict["connectors"] = connectors
        if floor_level is not UNSET:
            field_dict["floor_level"] = floor_level
        if coordinates is not UNSET:
            field_dict["coordinates"] = coordinates
        if physical_reference is not UNSET:
            field_dict["physical_reference"] = physical_reference
        if directions is not UNSET:
            field_dict["directions"] = directions
        if parking_restrictions is not UNSET:
            field_dict["parking_restrictions"] = parking_restrictions
        if images is not UNSET:
            field_dict["images"] = images
        if last_updated is not UNSET:
            field_dict["last_updated"] = last_updated

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.display_text_dto import DisplayTextDto
        from ..models.image_dto import ImageDto
        from ..models.geo_location_dto import GeoLocationDto
        from ..models.status_schedule_dto import StatusScheduleDto
        from ..models.connector_dto import ConnectorDto

        d = src_dict.copy()
        id = d.pop("id", UNSET)

        chargepointid = d.pop("chargepointid", UNSET)

        latitude = d.pop("latitude", UNSET)

        longitude = d.pop("longitude", UNSET)

        uid = d.pop("uid", UNSET)

        evse_id = d.pop("evse_id", UNSET)

        _status = d.pop("status", UNSET)
        status: Union[Unset, LocationEVSEDtoStatus]
        if isinstance(_status, Unset) or _status is None:
            status = UNSET
        else:
            status = LocationEVSEDtoStatus(_status)

        status_schedule = []
        _status_schedule = d.pop("status_schedule", UNSET)
        for status_schedule_item_data in _status_schedule or []:
            status_schedule_item = StatusScheduleDto.from_dict(
                status_schedule_item_data
            )

            status_schedule.append(status_schedule_item)

        capabilities = []
        _capabilities = d.pop("capabilities", UNSET)
        for capabilities_item_data in _capabilities or []:
            capabilities_item = LocationEVSEDtoCapabilitiesItem(capabilities_item_data)

            capabilities.append(capabilities_item)

        connectors = []
        _connectors = d.pop("connectors", UNSET)
        for connectors_item_data in _connectors or []:
            connectors_item = ConnectorDto.from_dict(connectors_item_data)

            connectors.append(connectors_item)

        floor_level = d.pop("floor_level", UNSET)

        _coordinates = d.pop("coordinates", UNSET)
        coordinates: Union[Unset, GeoLocationDto]
        if isinstance(_coordinates, Unset) or _coordinates is None:
            coordinates = UNSET
        else:
            coordinates = GeoLocationDto.from_dict(_coordinates)

        physical_reference = d.pop("physical_reference", UNSET)

        directions = []
        _directions = d.pop("directions", UNSET)
        for directions_item_data in _directions or []:
            directions_item = DisplayTextDto.from_dict(directions_item_data)

            directions.append(directions_item)

        parking_restrictions = []
        _parking_restrictions = d.pop("parking_restrictions", UNSET)
        for parking_restrictions_item_data in _parking_restrictions or []:
            parking_restrictions_item = LocationEVSEDtoParkingRestrictionsItem(
                parking_restrictions_item_data
            )

            parking_restrictions.append(parking_restrictions_item)

        images = []
        _images = d.pop("images", UNSET)
        for images_item_data in _images or []:
            images_item = ImageDto.from_dict(images_item_data)

            images.append(images_item)

        _last_updated = d.pop("last_updated", UNSET)
        last_updated: Union[Unset, datetime.datetime]
        if isinstance(_last_updated, Unset) or _last_updated is None:
            last_updated = UNSET
        else:
            last_updated = isoparse(_last_updated)

        location_evse_dto = cls(
            id=id,
            chargepointid=chargepointid,
            latitude=latitude,
            longitude=longitude,
            uid=uid,
            evse_id=evse_id,
            status=status,
            status_schedule=status_schedule,
            capabilities=capabilities,
            connectors=connectors,
            floor_level=floor_level,
            coordinates=coordinates,
            physical_reference=physical_reference,
            directions=directions,
            parking_restrictions=parking_restrictions,
            images=images,
            last_updated=last_updated,
        )

        location_evse_dto.additional_properties = d
        return location_evse_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class CdrLocationDtoPowerType(str, Enum):
    AC_1_PHASE = "AC_1_PHASE"
    AC_3_PHASE = "AC_3_PHASE"
    DC = "DC"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union

if TYPE_CHECKING:
    from ..models.regular_hours_dto import RegularHoursDto
    from ..models.exceptional_period_dto import ExceptionalPeriodDto


T = TypeVar("T", bound="HoursDto")


@_attrs_define
class HoursDto:
    """
    Attributes:
        twentyfourseven (Union[Unset, bool]):
        regular_hours (Union[Unset, List['RegularHoursDto']]):
        exceptional_openings (Union[Unset, List['ExceptionalPeriodDto']]):
        exceptional_closings (Union[Unset, List['ExceptionalPeriodDto']]):
    """

    twentyfourseven: Union[Unset, bool] = UNSET
    regular_hours: Union[Unset, List["RegularHoursDto"]] = UNSET
    exceptional_openings: Union[Unset, List["ExceptionalPeriodDto"]] = UNSET
    exceptional_closings: Union[Unset, List["ExceptionalPeriodDto"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        twentyfourseven = self.twentyfourseven

        regular_hours: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.regular_hours, Unset):
            regular_hours = []
            for regular_hours_item_data in self.regular_hours:
                regular_hours_item = regular_hours_item_data.to_dict()
                regular_hours.append(regular_hours_item)

        exceptional_openings: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.exceptional_openings, Unset):
            exceptional_openings = []
            for exceptional_openings_item_data in self.exceptional_openings:
                exceptional_openings_item = exceptional_openings_item_data.to_dict()
                exceptional_openings.append(exceptional_openings_item)

        exceptional_closings: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.exceptional_closings, Unset):
            exceptional_closings = []
            for exceptional_closings_item_data in self.exceptional_closings:
                exceptional_closings_item = exceptional_closings_item_data.to_dict()
                exceptional_closings.append(exceptional_closings_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if twentyfourseven is not UNSET:
            field_dict["twentyfourseven"] = twentyfourseven
        if regular_hours is not UNSET:
            field_dict["regular_hours"] = regular_hours
        if exceptional_openings is not UNSET:
            field_dict["exceptional_openings"] = exceptional_openings
        if exceptional_closings is not UNSET:
            field_dict["exceptional_closings"] = exceptional_closings

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.regular_hours_dto import RegularHoursDto
        from ..models.exceptional_period_dto import ExceptionalPeriodDto

        d = src_dict.copy()
        twentyfourseven = d.pop("twentyfourseven", UNSET)

        regular_hours = []
        _regular_hours = d.pop("regular_hours", UNSET)
        for regular_hours_item_data in _regular_hours or []:
            regular_hours_item = RegularHoursDto.from_dict(regular_hours_item_data)

            regular_hours.append(regular_hours_item)

        exceptional_openings = []
        _exceptional_openings = d.pop("exceptional_openings", UNSET)
        for exceptional_openings_item_data in _exceptional_openings or []:
            exceptional_openings_item = ExceptionalPeriodDto.from_dict(
                exceptional_openings_item_data
            )

            exceptional_openings.append(exceptional_openings_item)

        exceptional_closings = []
        _exceptional_closings = d.pop("exceptional_closings", UNSET)
        for exceptional_closings_item_data in _exceptional_closings or []:
            exceptional_closings_item = ExceptionalPeriodDto.from_dict(
                exceptional_closings_item_data
            )

            exceptional_closings.append(exceptional_closings_item)

        hours_dto = cls(
            twentyfourseven=twentyfourseven,
            regular_hours=regular_hours,
            exceptional_openings=exceptional_openings,
            exceptional_closings=exceptional_closings,
        )

        hours_dto.additional_properties = d
        return hours_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class EnvironmentalImpactDtoCategory(str, Enum):
    CARBON_DIOXIDE = "CARBON_DIOXIDE"
    NUCLEAR_WASTE = "NUCLEAR_WASTE"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class CdrStartedByTokenDtoTokenType(str, Enum):
    AD_HOC_USER = "AD_HOC_USER"
    APP_USER = "APP_USER"
    OTHER = "OTHER"
    RFID = "RFID"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class IdTagInfoStatus(str, Enum):
    ACCEPTED = "Accepted"
    BLOCKED = "Blocked"
    CONCURRENTTX = "ConcurrentTx"
    EXPIRED = "Expired"
    INVALID = "Invalid"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from dateutil.parser import isoparse
import datetime
from typing import Union

if TYPE_CHECKING:
    from ..models.tariff_distribution_history_dto import TariffDistributionHistoryDto


T = TypeVar("T", bound="TariffDistributionGetDto")


@_attrs_define
class TariffDistributionGetDto:
    """
    Attributes:
        id (str):
        name (Union[Unset, str]):
        ou_code (Union[Unset, str]):
        energy_compensation (Union[Unset, float]):
        fixed_tenant_k_wh_fee (Union[Unset, float]):
        percentage_fee_customer (Union[Unset, float]):
        percentage_fee_tenant (Union[Unset, float]):
        price_history (Union[Unset, List['TariffDistributionHistoryDto']]):
        created (Union[Unset, datetime.datetime]):
        deleted (Union[Unset, datetime.datetime]):
        updated (Union[Unset, datetime.datetime]):
    """

    id: str
    name: Union[Unset, str] = UNSET
    ou_code: Union[Unset, str] = UNSET
    energy_compensation: Union[Unset, float] = UNSET
    fixed_tenant_k_wh_fee: Union[Unset, float] = UNSET
    percentage_fee_customer: Union[Unset, float] = UNSET
    percentage_fee_tenant: Union[Unset, float] = UNSET
    price_history: Union[Unset, List["TariffDistributionHistoryDto"]] = UNSET
    created: Union[Unset, datetime.datetime] = UNSET
    deleted: Union[Unset, datetime.datetime] = UNSET
    updated: Union[Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        name = self.name

        ou_code = self.ou_code

        energy_compensation = self.energy_compensation

        fixed_tenant_k_wh_fee = self.fixed_tenant_k_wh_fee

        percentage_fee_customer = self.percentage_fee_customer

        percentage_fee_tenant = self.percentage_fee_tenant

        price_history: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.price_history, Unset):
            price_history = []
            for price_history_item_data in self.price_history:
                price_history_item = price_history_item_data.to_dict()
                price_history.append(price_history_item)

        created: Union[Unset, str] = UNSET
        if not isinstance(self.created, Unset):
            created = self.created.isoformat()

        deleted: Union[Unset, str] = UNSET
        if not isinstance(self.deleted, Unset):
            deleted = self.deleted.isoformat()

        updated: Union[Unset, str] = UNSET
        if not isinstance(self.updated, Unset):
            updated = self.updated.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
            }
        )
        if name is not UNSET:
            field_dict["name"] = name
        if ou_code is not UNSET:
            field_dict["ouCode"] = ou_code
        if energy_compensation is not UNSET:
            field_dict["energyCompensation"] = energy_compensation
        if fixed_tenant_k_wh_fee is not UNSET:
            field_dict["fixedTenantKWhFee"] = fixed_tenant_k_wh_fee
        if percentage_fee_customer is not UNSET:
            field_dict["percentageFeeCustomer"] = percentage_fee_customer
        if percentage_fee_tenant is not UNSET:
            field_dict["percentageFeeTenant"] = percentage_fee_tenant
        if price_history is not UNSET:
            field_dict["priceHistory"] = price_history
        if created is not UNSET:
            field_dict["created"] = created
        if deleted is not UNSET:
            field_dict["deleted"] = deleted
        if updated is not UNSET:
            field_dict["updated"] = updated

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.tariff_distribution_history_dto import (
            TariffDistributionHistoryDto,
        )

        d = src_dict.copy()
        id = d.pop("id")

        name = d.pop("name", UNSET)

        ou_code = d.pop("ouCode", UNSET)

        energy_compensation = d.pop("energyCompensation", UNSET)

        fixed_tenant_k_wh_fee = d.pop("fixedTenantKWhFee", UNSET)

        percentage_fee_customer = d.pop("percentageFeeCustomer", UNSET)

        percentage_fee_tenant = d.pop("percentageFeeTenant", UNSET)

        price_history = []
        _price_history = d.pop("priceHistory", UNSET)
        for price_history_item_data in _price_history or []:
            price_history_item = TariffDistributionHistoryDto.from_dict(
                price_history_item_data
            )

            price_history.append(price_history_item)

        _created = d.pop("created", UNSET)
        created: Union[Unset, datetime.datetime]
        if isinstance(_created, Unset) or _created is None:
            created = UNSET
        else:
            created = isoparse(_created)

        _deleted = d.pop("deleted", UNSET)
        deleted: Union[Unset, datetime.datetime]
        if isinstance(_deleted, Unset) or _deleted is None:
            deleted = UNSET
        else:
            deleted = isoparse(_deleted)

        _updated = d.pop("updated", UNSET)
        updated: Union[Unset, datetime.datetime]
        if isinstance(_updated, Unset) or _updated is None:
            updated = UNSET
        else:
            updated = isoparse(_updated)

        tariff_distribution_get_dto = cls(
            id=id,
            name=name,
            ou_code=ou_code,
            energy_compensation=energy_compensation,
            fixed_tenant_k_wh_fee=fixed_tenant_k_wh_fee,
            percentage_fee_customer=percentage_fee_customer,
            percentage_fee_tenant=percentage_fee_tenant,
            price_history=price_history,
            created=created,
            deleted=deleted,
            updated=updated,
        )

        tariff_distribution_get_dto.additional_properties = d
        return tariff_distribution_get_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar, TYPE_CHECKING

from typing import List


from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from typing import Union

if TYPE_CHECKING:
    from ..models.longship_error_detail import LongshipErrorDetail


T = TypeVar("T", bound="LongshipError")


@_attrs_define
class LongshipError:
    """
    Attributes:
        code (Union[Unset, str]): The error code.
        error_details (Union[Unset, LongshipErrorDetail]): More details about the error.
    """

    code: Union[Unset, str] = UNSET
    error_details: Union[Unset, "LongshipErrorDetail"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        code = self.code

        error_details: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.error_details, Unset):
            error_details = self.error_details.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if code is not UNSET:
            field_dict["code"] = code
        if error_details is not UNSET:
            field_dict["errorDetails"] = error_details

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.longship_error_detail import LongshipErrorDetail

        d = src_dict.copy()
        code = d.pop("code", UNSET)

        _error_details = d.pop("errorDetails", UNSET)
        error_details: Union[Unset, LongshipErrorDetail]
        if isinstance(_error_details, Unset) or _error_details is None:
            error_details = UNSET
        else:
            error_details = LongshipErrorDetail.from_dict(_error_details)

        longship_error = cls(
            code=code,
            error_details=error_details,
        )

        longship_error.additional_properties = d
        return longship_error

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
